@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,47 +23,65 @@ import logging
 import numpy as np
 
 from rlgraph import get_backend
-from rlgraph.components import Component, Exploration, PreprocessorStack, NeuralNetwork, Synchronizable, Policy, \
-    Optimizer
+from rlgraph.components import Component, Exploration, PreprocessorStack, Synchronizable, Policy, Optimizer, \
+    ValueFunction, DictMerger, ContainerSplitter
 from rlgraph.graphs.graph_builder import GraphBuilder
 from rlgraph.graphs.graph_executor import GraphExecutor
 from rlgraph.spaces import Space, ContainerSpace
-from rlgraph.utils.decorators import rlgraph_api
+from rlgraph.utils.decorators import rlgraph_api, graph_fn
 from rlgraph.utils.input_parsing import parse_execution_spec, parse_observe_spec, parse_update_spec
 from rlgraph.utils.specifiable import Specifiable
 
+if get_backend() == "tf":
+    import tensorflow as tf
+
 
 class Agent(Specifiable):
     """
     Generic agent defining RLGraph-API operations and parses and sanitizes configuration specs.
     """
+
     def __init__(self, state_space, action_space, discount=0.98,
                  preprocessing_spec=None, network_spec=None, internal_states_space=None,
-                 policy_spec=None,
-                 exploration_spec=None, execution_spec=None, optimizer_spec=None, observe_spec=None, update_spec=None,
+                 policy_spec=None, value_function_spec=None,
+                 exploration_spec=None, execution_spec=None, optimizer_spec=None, value_function_optimizer_spec=None,
+                 observe_spec=None, update_spec=None,
                  summary_spec=None, saver_spec=None, auto_build=True, name="agent"):
         """
         Args:
             state_space (Union[dict,Space]): Spec dict for the state Space or a direct Space object.
             action_space (Union[dict,Space]): Spec dict for the action Space or a direct Space object.
+
             preprocessing_spec (Optional[list,PreprocessorStack]): The spec list for the different necessary states
                 preprocessing steps or a PreprocessorStack object itself.
+
             discount (float): The discount factor (gamma).
+
             network_spec (Optional[list,NeuralNetwork]): Spec list for a NeuralNetwork Component or the NeuralNetwork
                 object itself.
+
             internal_states_space (Optional[Union[dict,Space]]): Spec dict for the internal-states Space or a direct
                 Space object for the Space(s) of the internal (RNN) states.
+
             policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
+            value_function_spec (list): Neural network specification for baseline.
+
             exploration_spec (Optional[dict]): The spec-dict to create the Exploration Component.
             execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
             optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
+
+            value_function_optimizer_spec (dict): Optimizer config for value function otpimizer. If None, the optimizer
+                spec for the policy is used (same learning rate and optimizer type).
+
             observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
             update_spec (Optional[dict]): Spec-dict to specify `Agent.update()` settings.
             summary_spec (Optional[dict]): Spec-dict to specify summary settings.
             saver_spec (Optional[dict]): Spec-dict to specify saver settings.
+
             auto_build (Optional[bool]): If True (default), immediately builds the graph using the agent's
                 graph builder. If false, users must separately call agent.build(). Useful for debugging or analyzing
                 components before building.
+
             name (str): Some name for this Agent object.
         """
         super(Agent, self).__init__()
@@ -81,12 +99,13 @@ class Agent(Specifiable):
         self.logger.info("Parsed action space definition: {}".format(self.action_space))
 
         self.discount = discount
+        self.build_options = {}
 
         # The agent's root-Component.
-        self.root_component = Component(name=self.name)
+        self.root_component = Component(name=self.name, nesting_level=0)
 
         # Define the input-Spaces:
-        # Tag the input-Space to `self.set_policy_weights` as equal to whatever the variables-Space will be for
+        # Tag the input-Space to `self.set_weights` as equal to whatever the variables-Space will be for
         # the Agent's policy Component.
         self.input_spaces = dict(
             states=self.state_space.with_batch_rank(),
@@ -104,7 +123,7 @@ class Agent(Specifiable):
 
         # Construct the Policy network.
         policy_spec = policy_spec or dict()
-        if network_spec is not None:
+        if "network_spec" not in policy_spec:
             policy_spec["network_spec"] = network_spec
         if "action_space" not in policy_spec:
             policy_spec["action_space"] = self.action_space
@@ -114,6 +133,17 @@ class Agent(Specifiable):
         # Done by default.
         self.policy.add_components(Synchronizable(), expose_apis="sync")
 
+        # Create non-shared baseline network.
+        self.value_function = None
+        if value_function_spec is not None:
+            self.value_function = ValueFunction(network_spec=value_function_spec)
+            self.value_function.add_components(Synchronizable(), expose_apis="sync")
+            self.vars_merger = DictMerger("policy", "vf", scope="variable-dict-merger")
+            self.vars_splitter = ContainerSplitter("policy", "vf", scope="variable-container-splitter")
+        else:
+            self.vars_merger = DictMerger("policy", scope="variable-dict-merger")
+            self.vars_splitter = ContainerSplitter("policy", scope="variable-container-splitter")
+
         self.internal_states_space = Space.from_spec(internal_states_space)
 
         # An object implementing the loss function interface is only strictly needed
@@ -133,11 +163,11 @@ class Agent(Specifiable):
                 return []
             return tuple([[] for _ in range(i)])
 
-        self.states_buffer = defaultdict(list)  #partial(fact_, len(self.flat_state_space)))
+        self.states_buffer = defaultdict(list)  # partial(fact_, len(self.flat_state_space)))
         self.actions_buffer = defaultdict(partial(factory_, len(self.flat_action_space or [])))
         self.internals_buffer = defaultdict(list)
         self.rewards_buffer = defaultdict(list)
-        self.next_states_buffer = defaultdict(list)  #partial(fact_, len(self.flat_state_space)))
+        self.next_states_buffer = defaultdict(list)  # partial(fact_, len(self.flat_state_space)))
         self.terminals_buffer = defaultdict(list)
 
         self.observe_spec = parse_observe_spec(observe_spec)
@@ -151,8 +181,16 @@ class Agent(Specifiable):
             # Save spec in case agent needs to create more optimizers e.g. for baseline.
             self.optimizer_spec = optimizer_spec
             self.optimizer = Optimizer.from_spec(optimizer_spec)
-            #get_optimizer_from_device_strategy(
-                #optimizer_spec, self.execution_spec.get("device_strategy", 'default')
+
+        self.value_function_optimizer = None
+        if self.value_function is not None:
+            if value_function_optimizer_spec is None:
+                vf_optimizer_spec = self.optimizer_spec
+            else:
+                vf_optimizer_spec = value_function_optimizer_spec
+            vf_optimizer_spec["scope"] = "value-function-optimizer"
+            self.value_function_optimizer = Optimizer.from_spec(vf_optimizer_spec)
+
         # Update-spec dict tells the Agent how to update (e.g. memory batch size).
         self.update_spec = parse_update_spec(update_spec)
 
@@ -175,40 +213,62 @@ class Agent(Specifiable):
         if env_id is None:
             env_id = self.default_env
         del self.states_buffer[env_id]  # = ([] for _ in range(len(self.flat_state_space)))
-        del self.actions_buffer[env_id]   # = ([] for _ in range(len(self.flat_action_space)))
+        del self.actions_buffer[env_id]  # = ([] for _ in range(len(self.flat_action_space)))
         del self.internals_buffer[env_id]  # = []
         del self.rewards_buffer[env_id]  # = []
         del self.next_states_buffer[env_id]  # = ([] for _ in range(len(self.flat_state_space)))
         del self.terminals_buffer[env_id]  # = []
 
-    # TODO optimizer scope missing?
-    def define_graph_api(self, policy_scope, pre_processor_scope, *params):
+    def define_graph_api(self, *args, **kwargs):
         """
         Can be used to specify and then `self.define_api_method` the Agent's CoreComponent's API methods.
         Each agent implements this to build its algorithm logic.
-
-        Args:
-            policy_scope (str): The global scope of the Policy within the Agent.
-            pre_processor_scope (str): The global scope of the PreprocessorStack within the Agent.
-            params (any): Params to be used freely by child Agent implementations.
         """
-        # Add api methods for syncing.
+        agent = self
+
+        if self.value_function is not None:
+            # This avoids variable-incompleteness for the value-function component in a multi-GPU setup, where the root
+            # value-function never performs any forward pass (only used as variable storage).
+            @rlgraph_api(component=self.root_component)
+            def get_state_values(root, preprocessed_states):
+                vf = root.get_sub_component_by_name(agent.value_function.scope)
+                return vf.value_output(preprocessed_states)
+
+        # Add API methods for syncing.
         @rlgraph_api(component=self.root_component)
-        def get_policy_weights(self):
-            policy = self.get_sub_component_by_name(policy_scope)
-            return policy._variables()
+        def get_weights(root):
+            policy = root.get_sub_component_by_name(agent.policy.scope)
+            policy_weights = policy._variables()
+            value_function_weights = None
+            if agent.value_function is not None:
+                value_func = root.get_sub_component_by_name(agent.value_function.scope)
+                value_function_weights = value_func._variables()
+            return dict(policy_weights=policy_weights, value_function_weights=value_function_weights)
 
         @rlgraph_api(component=self.root_component, must_be_complete=False)
-        def set_policy_weights(self, weights):
-            policy = self.get_sub_component_by_name(policy_scope)
-            return policy.sync(weights)
+        def set_weights(root, policy_weights, value_function_weights=None):
+            policy = root.get_sub_component_by_name(agent.policy.scope)
+            policy_sync_op = policy.sync(policy_weights)
+            if value_function_weights is not None:
+                assert agent.value_function is not None
+                vf = root.get_sub_component_by_name(agent.value_function.scope)
+                vf_sync_op = vf.sync(value_function_weights)
+                return root._graph_fn_group(policy_sync_op, vf_sync_op)
+            else:
+                return policy_sync_op
+
+        # TODO: Replace this with future on-the-fly-API-components.
+        @graph_fn(component=self.root_component)
+        def _graph_fn_group(root, *ops):
+            if get_backend() == "tf":
+                return tf.group(*ops)
+            return ops[0]
 
         # To pre-process external data if needed.
         @rlgraph_api(component=self.root_component)
-        def preprocess_states(self, states):
-            preprocessor_stack = self.get_sub_component_by_name(pre_processor_scope)
-            preprocessed_states = preprocessor_stack.preprocess(states)
-            return preprocessed_states
+        def preprocess_states(root, states):
+            preprocessor_stack = root.get_sub_component_by_name(agent.preprocessor.scope)
+            return preprocessor_stack.preprocess(states)
 
     def _build_graph(self, root_components, input_spaces, **kwargs):
         """
@@ -224,14 +284,16 @@ class Agent(Specifiable):
         Args:
             build_options (Optional[dict]): Optional build options, see build doc.
         """
-        assert not self.graph_built,\
+        if build_options is not None:
+            self.build_options.update(build_options)
+        assert not self.graph_built, \
             "ERROR: Attempting to build agent which has already been built. Ensure auto_build parameter is set to " \
             "False (was {}), and method has not been called twice".format(self.auto_build)
 
         # TODO let agent have a list of root-components
         return self._build_graph(
             [self.root_component], self.input_spaces, optimizer=self.optimizer,
-            build_options=build_options, batch_size=self.update_spec["batch_size"]
+            build_options=self.build_options, batch_size=self.update_spec["batch_size"]
         )
 
     def preprocess_states(self, states):
@@ -351,11 +413,9 @@ class Agent(Specifiable):
                 self.terminals_buffer[env_id][-1] = True
 
                 # TODO: Apply n-step post-processing if necessary.
-                # if self.observe_spec["n_step"] > 1:
-                #    pass
-
                 if self.flat_action_space is not None:
-                    actions_ = {key: np.asarray(self.actions_buffer[env_id][i]) for i, key in enumerate(self.flat_action_space.keys())}
+                    actions_ = {key: np.asarray(self.actions_buffer[env_id][i]) for i, key in
+                                enumerate(self.flat_action_space.keys())}
                 else:
                     actions_ = np.asarray(self.actions_buffer[env_id])
                 self._observe_graph(
@@ -368,6 +428,13 @@ class Agent(Specifiable):
                 )
                 self.reset_env_buffers(env_id)
         else:
+            if not batched:
+                preprocessed_states = self.preprocessed_state_space.force_batch(preprocessed_states)
+                next_states = self.preprocessed_state_space.force_batch(next_states)
+                actions = self.action_space.force_batch(actions)
+                rewards = [rewards]
+                terminals = [terminals]
+
             self._observe_graph(preprocessed_states, actions, internals, rewards, next_states, terminals)
 
     def _observe_graph(self, preprocessed_states, actions, internals, rewards, next_states, terminals):
@@ -388,7 +455,7 @@ class Agent(Specifiable):
         """
         raise NotImplementedError
 
-    def update(self, batch=None):
+    def update(self, batch=None, **kwargs):
         """
         Performs an update on the computation graph either via externally experience or
         by sampling from an internal memory.
@@ -464,33 +531,65 @@ class Agent(Specifiable):
         """
         self.graph_executor.store_model(path=path, add_timestep=add_timestep)
 
-    def load_model(self, path=None):
+    def load_model(self, checkpoint_directory=None, checkpoint_path=None):
         """
-        Load model from serialized format.
+        Loads model from specified path location using the following semantics:
+
+        If checkpoint directory and checkpoint path are given, attempts to find `checkpoint_path` as relative path from
+        `checkpoint_directory`.
+
+        If a checkpoint directory is given but no path (e.g. because timestep of checkpoint is not known in advance),
+        attempts to fetch latest check-point.
+
+        If no directory is given, attempts to fetch checkpoint from the full absolute path `checkpoint_path'.
 
         Args:
-            path (str): Path to checkpoint directory.
+            checkpoint_directory (str): Optional path to directory containing checkpoint(s).
+            checkpoint_path (str): Path to specific model checkpoint.
         """
-        self.graph_executor.load_model(path=path)
+        self.graph_executor.load_model(checkpoint_directory=checkpoint_directory, checkpoint_path=checkpoint_path)
 
-    def get_policy_weights(self):
+    def get_weights(self):
         """
         Returns all weights relevant for the agent's policy for syncing purposes.
 
         Returns:
             any: Weights and optionally weight meta data for this model.
         """
-        return dict(self.graph_executor.execute("get_policy_weights"))
+        return self.graph_executor.execute("get_weights")
 
-    def set_policy_weights(self, weights):
+    def set_weights(self, policy_weights, value_function_weights=None):
         """
         Sets policy weights of this agent, e.g. for external syncing purposes.
 
         Args:
-            weights (any): Weights and optionally meta data to update depending on the backend.
+            policy_weights (any): Weights and optionally meta data to update depending on the backend.
+            value_function_weights (Optional[any]): Optional value function weights.
 
         Raises:
             ValueError if weights do not match graph weights in shapes and types.
         """
-        return self.graph_executor.execute(("set_policy_weights", weights))
+        # TODO generic *args here and specific names in specific agents?
+        if value_function_weights is not None:
+            return self.graph_executor.execute(("set_weights", [policy_weights, value_function_weights]))
+        else:
+            return self.graph_executor.execute(("set_weights", policy_weights))
 
+    def post_process(self, batch):
+        """
+        Optional method to post-processes a batch if post-processing is off-loaded to workers instead of
+        executed by a central learner before computing the loss.
+
+        The post-processing function must be able to post-process batches of multiple environments
+        and episodes with non-terminated fragments via sequence-indices.
+
+        This enables efficient processing of multi-environment batches.
+
+        Args:
+            batch (dict): Batch to process. Must contain key 'sequence-indices' to describe where
+                environment fragments end (even if the corresponding episode has not terminated.
+
+        Returns:
+            any: Post-processed batch.
+        """
+        pass
