@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,8 +36,8 @@ class DQNAgent(Agent):
     [3] Dueling Network Architectures for Deep Reinforcement Learning, Wang et al. - 2016
     [4] https://en.wikipedia.org/wiki/Huber_loss
     """
-    def __init__(self, double_q=True, dueling_q=True, huber_loss=False, n_step=1, memory_spec=None,
-                 store_last_memory_batch=False, store_last_q_table=False, **kwargs):
+    def __init__(self, double_q=True, dueling_q=True, huber_loss=False, n_step=1, shared_container_action_target=True,
+                 memory_spec=None, store_last_memory_batch=False, store_last_q_table=False, **kwargs):
         """
         Args:
             double_q (bool): Whether to use the double DQN loss function (see [2]).
@@ -65,6 +65,10 @@ class DQNAgent(Agent):
             policy_spec=policy_spec, name=kwargs.pop("name", "dqn-agent"), **kwargs
         )
 
+        # TODO: Have to manually set it here for multi-GPU synchronizer to know its number
+        # TODO: of return values when calling _graph_fn_calculate_update_from_external_batch.
+        #self.root_component.graph_fn_num_outputs["_graph_fn_update_from_external_batch"] = 4
+
         # Assert that the synch interval is a multiple of the update_interval.
         if self.update_spec["sync_interval"] / self.update_spec["update_interval"] != \
                 self.update_spec["sync_interval"] // self.update_spec["update_interval"]:
@@ -76,6 +80,7 @@ class DQNAgent(Agent):
         self.double_q = double_q
         self.dueling_q = dueling_q
         self.huber_loss = huber_loss
+        self.shared_container_action_target = shared_container_action_target
 
         # Debugging tools.
         self.store_last_memory_batch = store_last_memory_batch
@@ -92,7 +97,7 @@ class DQNAgent(Agent):
         self.input_spaces.update(dict(
             actions=self.action_space.with_batch_rank(),
             # weights will have a Space derived from the vars of policy.
-            weights="variables:{}".format(self.policy.scope),
+            policy_weights="variables:{}".format(self.policy.scope),
             time_step=int,
             use_exploration=bool,
             preprocessed_states=preprocessed_state_space,
@@ -101,11 +106,9 @@ class DQNAgent(Agent):
             next_states=preprocessed_state_space,
             preprocessed_next_states=preprocessed_state_space,
             importance_weights=weight_space,
-            # TODO: This is currently necessary for multi-GPU handling (as the update_from_external_batch
-            # TODO: gets overridden by a generic function with args=*inputs)
-            #inputs=[preprocessed_state_space, self.action_space.with_batch_rank(), reward_space, terminal_space,
-            #        preprocessed_state_space, weight_space]
         ))
+        if self.value_function is not None:
+            self.input_spaces["value_function_weights"] = "variables:{}".format(self.value_function.scope),
 
         # The merger to merge inputs into one record Dict going into the memory.
         self.merger = DictMerger("states", "actions", "rewards", "next_states", "terminals")
@@ -127,16 +130,18 @@ class DQNAgent(Agent):
         use_importance_weights = isinstance(self.memory, PrioritizedReplay)
         self.loss_function = DQNLossFunction(
             discount=self.discount, double_q=self.double_q, huber_loss=self.huber_loss,
+            shared_container_action_target=shared_container_action_target,
             importance_weights=use_importance_weights, n_step=n_step
         )
 
-        # Add all our sub-components to the core.
-        sub_components = [self.preprocessor, self.merger, self.memory, self.splitter, self.policy,
-                          self.target_policy, self.exploration, self.loss_function, self.optimizer]
-        self.root_component.add_components(*sub_components)
+        self.root_component.add_components(
+            self.preprocessor, self.merger, self.memory, self.splitter, self.policy, self.target_policy,
+            self.value_function, self.value_function_optimizer,  # <- should both be None for DQN
+            self.exploration, self.loss_function, self.optimizer, self.vars_merger, self.vars_splitter
+        )
 
         # Define the Agent's (root-Component's) API.
-        self.define_graph_api(self.policy.scope, self.preprocessor.scope, self.optimizer.scope, *sub_components)
+        self.define_graph_api()
 
         # markup = get_graph_markup(self.graph_builder.root_component)
         # print(markup)
@@ -145,76 +150,64 @@ class DQNAgent(Agent):
                               batch_size=self.update_spec["batch_size"])
             self.graph_built = True
 
-            # TODO: What should the external batch be? 0s.
-            #if "multi-gpu-sync-optimizer" in self.root_component.sub_components:
-            #    # Get 1st return op of API-method `calculate_update_from_external_batch`
-            #    # (which is the group of stage-ops).
-            #    stage_op = self.root_component.sub_components["multi-gpu-sync-optimizer"].\
-            #        api_methods["calculate_update_from_external_batch"].\
-            #        out_op_columns[0].op_records[0].op
-            #    # Initialize the stage.
-            #    self.graph_executor.monitored_session.run_step_fn(
-            #        lambda step_context: step_context.session.run(stage_op)
-            #    )
+    def define_graph_api(self, *args, **kwargs):
+        super(DQNAgent, self).define_graph_api()
 
-    def define_graph_api(self, policy_scope, pre_processor_scope, optimizer_scope, *sub_components):
-        super(DQNAgent, self).define_graph_api(policy_scope, pre_processor_scope)
-
-        preprocessor, merger, memory, splitter, policy, target_policy, exploration, loss_function, optimizer = \
-            sub_components
+        agent = self
 
         # Reset operation (resets preprocessor).
         if self.preprocessing_required:
             @rlgraph_api(component=self.root_component)
-            def reset_preprocessor(self):
-                reset_op = preprocessor.reset()
+            def reset_preprocessor(root):
+                reset_op = agent.preprocessor.reset()
                 return reset_op
 
         # Act from preprocessed states.
         @rlgraph_api(component=self.root_component)
-        def action_from_preprocessed_state(self, preprocessed_states, time_step=0, use_exploration=True):
-            sample_deterministic = policy.get_deterministic_action(preprocessed_states)
-            actions = exploration.get_action(sample_deterministic["action"], time_step, use_exploration)
-            return preprocessed_states, actions
+        def action_from_preprocessed_state(root, preprocessed_states, time_step=0, use_exploration=True):
+            sample_deterministic = agent.policy.get_deterministic_action(preprocessed_states)
+            actions = agent.exploration.get_action(sample_deterministic["action"], time_step, use_exploration)
+            return actions, preprocessed_states
 
         # State (from environment) to action with preprocessing.
         @rlgraph_api(component=self.root_component)
-        def get_preprocessed_state_and_action(self, states, time_step=0, use_exploration=True):
-            preprocessed_states = preprocessor.preprocess(states)
-            return self.action_from_preprocessed_state(preprocessed_states, time_step, use_exploration)
+        def get_preprocessed_state_and_action(root, states, time_step=0, use_exploration=True):
+            preprocessed_states = agent.preprocessor.preprocess(states)
+            return root.action_from_preprocessed_state(preprocessed_states, time_step, use_exploration)
 
         # Insert into memory.
         @rlgraph_api(component=self.root_component)
-        def insert_records(self, preprocessed_states, actions, rewards, next_states, terminals):
-            records = merger.merge(preprocessed_states, actions, rewards, next_states, terminals)
-            return memory.insert_records(records)
+        def insert_records(root, preprocessed_states, actions, rewards, next_states, terminals):
+            records = agent.merger.merge(preprocessed_states, actions, rewards, next_states, terminals)
+            return agent.memory.insert_records(records)
 
         # Syncing target-net.
         @rlgraph_api(component=self.root_component)
-        def sync_target_qnet(self):
+        def sync_target_qnet(root):
             # If we are a multi-GPU root:
             # Simply feeds everything into the multi-GPU sync optimizer's method and return.
-            if "multi-gpu-sync-optimizer" in self.sub_components:
-                multi_gpu_syncer = self.sub_components["multi-gpu-sync-optimizer"]
+            if "multi-gpu-synchronizer" in root.sub_components:
+                multi_gpu_syncer = root.sub_components["multi-gpu-synchronizer"]
                 return multi_gpu_syncer.sync_target_qnets()
+            # We could be the main root or a multi-GPU tower.
             else:
-                policy_vars = self.get_sub_component_by_name(policy_scope)._variables()
-                return self.get_sub_component_by_name("target-policy").sync(policy_vars)
+                policy_vars = root.get_sub_component_by_name(agent.policy.scope)._variables()
+                return root.get_sub_component_by_name(agent.target_policy.scope).sync(policy_vars)
 
         # Learn from memory.
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(self_):
+        def update_from_memory(root):
             # Non prioritized memory will just return weight 1.0 for all samples.
-            records, sample_indices, importance_weights = memory.get_records(self.update_spec["batch_size"])
-            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = splitter.split(records)
+            records, sample_indices, importance_weights = agent.memory.get_records(agent.update_spec["batch_size"])
+            preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.split(records)
 
-            step_op, loss, loss_per_item, q_values_s = self_.update_from_external_batch(
+            step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
                 preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights
             )
 
             # TODO this is really annoying.. will be solved once we have dict returns.
-            if isinstance(memory, PrioritizedReplay):
-                update_pr_step_op = memory.update_records(sample_indices, loss_per_item)
+            if isinstance(agent.memory, PrioritizedReplay):
+                update_pr_step_op = agent.memory.update_records(sample_indices, loss_per_item)
                 return step_op, loss, loss_per_item, records, q_values_s, update_pr_step_op
             else:
                 return step_op, loss, loss_per_item, records, q_values_s
@@ -222,59 +215,67 @@ class DQNAgent(Agent):
         # Learn from an external batch.
         @rlgraph_api(component=self.root_component)
         def update_from_external_batch(
-                self_, preprocessed_states, actions, rewards, terminals, preprocessed_next_states, importance_weights
+                root, preprocessed_states, actions, rewards, terminals, preprocessed_next_states,
+                importance_weights
         ):
             # If we are a multi-GPU root:
             # Simply feeds everything into the multi-GPU sync optimizer's method and return.
-            if "multi-gpu-sync-optimizer" in self_.sub_components:
-                main_policy_vars = self_.get_sub_component_by_name(policy_scope)._variables()
-                # TODO: This may be called differently in other agents (replace by root-policy).
-                grads_and_vars, loss, loss_per_item, q_values_s = \
-                    self_.sub_components["multi-gpu-sync-optimizer"].calculate_update_from_external_batch(
-                        main_policy_vars, preprocessed_states, actions, rewards, terminals, preprocessed_next_states,
-                        importance_weights
-                    )
-                step_op = self_.get_sub_component_by_name(optimizer_scope).apply_gradients(grads_and_vars)
-                step_and_sync_op = self_.sub_components["multi-gpu-sync-optimizer"].sync_policy_weights_to_towers(
-                    step_op, main_policy_vars
+            if "multi-gpu-synchronizer" in root.sub_components:
+                main_policy_vars = agent.policy._variables()
+                all_vars = agent.vars_merger.merge(main_policy_vars)
+                out = root.sub_components["multi-gpu-synchronizer"].calculate_update_from_external_batch(
+                    all_vars, preprocessed_states, actions, rewards, terminals,
+                    preprocessed_next_states, importance_weights
                 )
-                return step_and_sync_op, loss, loss_per_item, q_values_s
+                avg_grads_and_vars = agent.vars_splitter.split(out["avg_grads_and_vars_by_component"])
+                step_op = agent.optimizer.apply_gradients(avg_grads_and_vars)
+                step_and_sync_op = root.sub_components["multi-gpu-synchronizer"].sync_variables_to_towers(
+                    step_op, all_vars
+                )
+                q_values_s = out["additional_return_0"]
+                return step_and_sync_op, out["loss"], out["loss_per_item"], q_values_s
+
+            # Get sub-components relative to the root (could be multi-GPU setup where root=some-tower).
+            policy = root.get_sub_component_by_name(agent.policy.scope)
+            target_policy = root.get_sub_component_by_name(agent.target_policy.scope)
+            loss_function = root.get_sub_component_by_name(agent.loss_function.scope)
+            optimizer = root.get_sub_component_by_name(agent.optimizer.scope)
+            vars_merger = root.get_sub_component_by_name(agent.vars_merger.scope)
 
             # Get the different Q-values.
-            q_values_s = self_.get_sub_component_by_name(policy_scope).get_logits_probabilities_log_probs(
-                preprocessed_states
-            )["logits"]
-            qt_values_sp = self_.get_sub_component_by_name(target_policy.scope).get_logits_probabilities_log_probs(
-                preprocessed_next_states
-            )["logits"]
+            q_values_s = policy.get_logits_probabilities_log_probs(preprocessed_states)["logits"]
+            qt_values_sp = target_policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
 
             q_values_sp = None
             if self.double_q:
-                q_values_sp = self_.get_sub_component_by_name(policy_scope).get_logits_probabilities_log_probs(
-                    preprocessed_next_states
-                )["logits"]
+                q_values_sp = policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
 
-            loss, loss_per_item = self_.get_sub_component_by_name(loss_function.scope).loss(
+            loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp, importance_weights
             )
 
             # Args are passed in again because some device strategies may want to split them to different devices.
-            policy_vars = self_.get_sub_component_by_name(policy_scope)._variables()
+            policy_vars = policy._variables()
 
             # TODO: for a fully automated multi-GPU strategy, we would have to make sure that:
             # TODO: - every agent (root_component) has an update_from_external_batch method
             # TODO: - this if check is somehow automated and not necessary anymore (local optimizer must be called with different API-method, not step)
-            if hasattr(self_, "is_multi_gpu_tower") and self_.is_multi_gpu_tower is True:
-                grads_and_vars = self_.get_sub_component_by_name(optimizer_scope).calculate_gradients(policy_vars, loss)
-                return grads_and_vars, loss, loss_per_item, q_values_s
+            if hasattr(root, "is_multi_gpu_tower") and root.is_multi_gpu_tower is True:
+                grads_and_vars = optimizer.calculate_gradients(policy_vars, loss)
+                grads_and_vars_by_component = vars_merger.merge(grads_and_vars)
+                return grads_and_vars_by_component, loss, loss_per_item, q_values_s
             else:
                 step_op, loss, loss_per_item = optimizer.step(policy_vars, loss, loss_per_item)
                 return step_op, loss, loss_per_item, q_values_s
 
-        # TODO for testing
         @rlgraph_api(component=self.root_component)
-        def get_td_loss(self_, preprocessed_states, actions, rewards,
+        def get_td_loss(root, preprocessed_states, actions, rewards,
                         terminals, preprocessed_next_states, importance_weights):
+
+            policy = root.get_sub_component_by_name(agent.policy.scope)
+            target_policy = root.get_sub_component_by_name(agent.target_policy.scope)
+            loss_function = root.get_sub_component_by_name(agent.loss_function.scope)
+
             # Get the different Q-values.
             q_values_s = policy.get_logits_probabilities_log_probs(preprocessed_states)["logits"]
             qt_values_sp = target_policy.get_logits_probabilities_log_probs(preprocessed_next_states)["logits"]
@@ -318,11 +319,10 @@ class DQNAgent(Agent):
         self.timesteps += batch_size
 
         # Control, which return value to "pull" (depending on `additional_returns`).
-        return_ops = [1, 0] if "preprocessed_states" in extra_returns else [1]
+        return_ops = [0, 1] if "preprocessed_states" in extra_returns else [0]  # 1=preprocessed_states, 0=action
         ret = self.graph_executor.execute((
             call_method,
             [batched_states, self.timesteps, use_exploration],
-            # 0=preprocessed_states, 1=action
             return_ops
         ))  #, flip_batch_with_dict_keys=isinstance(self.action_space, ContainerSpace))
         if remove_batch_rank:
@@ -399,5 +399,17 @@ class DQNAgent(Agent):
         if self.preprocessing_required and len(self.preprocessor.variables) > 0:
             self.graph_executor.execute("reset_preprocessor")
 
+    def post_process(self, batch):
+        batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"],
+                       batch["next_states"], batch["importance_weights"]]
+        ret = self.graph_executor.execute(("get_td_loss", batch_input))
+
+        # Remove unnecessary return dicts.
+        if isinstance(ret, dict):
+            ret = ret["get_td_loss"]
+
+        # Return [0]=total loss, [1]=loss-per-item
+        return ret[0], ret[1]
+
     def __repr__(self):
         return "DQNAgent(doubleQ={} duelingQ={})".format(self.double_q, self.dueling_q)
