@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,6 +17,8 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import copy
+
 from rlgraph.utils.decorators import rlgraph_api
 from rlgraph.utils import RLGraphError
 from rlgraph.agents.agent import Agent
@@ -28,7 +30,7 @@ from rlgraph.components.common.environment_stepper import EnvironmentStepper
 from rlgraph.components.layers.preprocessing.reshape import ReShape
 from rlgraph.components.layers.preprocessing.transpose import Transpose
 from rlgraph.components.neural_networks.actor_component import ActorComponent
-from rlgraph.components.neural_networks.dynamic_batching_policy import DynamicBatchingPolicy
+from rlgraph.components.policies.dynamic_batching_policy import DynamicBatchingPolicy
 from rlgraph.components.loss_functions.impala_loss_function import IMPALALossFunction
 from rlgraph.components.memories.fifo_queue import FIFOQueue
 from rlgraph.components.memories.queue_runner import QueueRunner
@@ -135,15 +137,14 @@ class IMPALAAgent(Agent):
         # Limit communication in distributed mode between each actor and the learner (never between actors).
         execution_spec = kwargs.pop("execution_spec", None)
         if execution_spec is not None and execution_spec.get("mode") == "distributed":
-            execution_spec["session_config"] = dict(
+            default_dict(execution_spec["session_config"], dict(
                 type="monitored-training-session",
                 allow_soft_placement=True,
-                log_device_placement=False,
                 device_filters=["/job:learner/task:0"] + (
                     ["/job:actor/task:{}".format(execution_spec["distributed_spec"]["task_index"])] if
                     self.type == "actor" else ["/job:learner/task:0"]
                 )
-            )
+            ))
             # If Actor, make non-chief in either case (even if task idx == 0).
             if self.type == "actor":
                 execution_spec["distributed_spec"]["is_chief"] = False
@@ -517,7 +518,7 @@ class SingleIMPALAAgent(IMPALAAgent):
                  feed_previous_action_through_nn=True, feed_previous_reward_through_nn=True,
                  weight_pg=None, weight_baseline=None, weight_entropy=None,
                  num_workers=1, worker_sample_size=100,
-                 dynamic_batching=False, **kwargs):
+                 dynamic_batching=False, visualize=False, **kwargs):
         """
         Args:
             discount (float): The discount factor gamma.
@@ -534,11 +535,13 @@ class SingleIMPALAAgent(IMPALAAgent):
             weight_pg (float): See IMPALALossFunction Component.
             weight_baseline (float): See IMPALALossFunction Component.
             weight_entropy (float): See IMPALALossFunction Component.
+            num_workers (int): How many actors (workers) should be run in separate threads.
             worker_sample_size (int): How many steps the actor will perform in the environment each sample-run.
             dynamic_batching (bool): Whether to use the deepmind's custom dynamic batching op for wrapping the
                 optimizer's step call. The batcher.so file must be compiled for this to work (see Docker file).
                 Default: False.
-            num_workers (int): How many actors (workers) should be run in separate threads.
+            visualize (Union[int,bool]): Whether and how many workers to visualize.
+                Default: False (no visualization).
         """
         # Now that we fixed the Agent's spec, call the super constructor.
         super(SingleIMPALAAgent, self).__init__(
@@ -558,6 +561,7 @@ class SingleIMPALAAgent(IMPALAAgent):
         )
         self.dynamic_batching = dynamic_batching
         self.num_workers = num_workers
+        self.visualize = visualize
 
         # If we use dynamic batching, wrap the dynamic batcher around the policy's graph_fn that we
         # actually call below during our build.
@@ -581,13 +585,7 @@ class SingleIMPALAAgent(IMPALAAgent):
         else:
             internal_states_slicer = None
 
-        # TODO: add state transposer, remove action/rewards transposer (part of state).
-        self.transpose_states = Transpose(scope="transpose-states")
-        self.transpose_terminals = Transpose(scope="transpose-terminals")
-        self.transpose_action_probs = Transpose(scope="transpose-a-probs-mu")
-        # Transposing of actions and rewards may not be needed.
-        self.transpose_actions = Transpose(scope="transpose-actions")
-        self.transpose_rewards = Transpose(scope="transpose-rewards")
+        self.transposer = Transpose(scope="transposer")
 
         # Create an IMPALALossFunction with some parameters.
         self.loss_function = IMPALALossFunction(
@@ -604,11 +602,21 @@ class SingleIMPALAAgent(IMPALAAgent):
 
         self.environment_steppers = list()
         for i in range(self.num_workers):
+            environment_spec_ = copy.deepcopy(environment_spec)
+            if self.visualize is True or (isinstance(self.visualize, int) and i+1 <= self.visualize):
+                environment_spec_["visualize"] = True
+
+            # Force worker_sample_size for IMPALA NNs (LSTM) in env-stepper to be 1.
+            policy_spec = copy.deepcopy(self.policy_spec)
+            if isinstance(policy_spec, dict) and isinstance(policy_spec["network_spec"], dict) and \
+                    "type" in policy_spec["network_spec"] and "IMPALANetwork" in policy_spec["network_spec"]["type"]:
+                policy_spec["network_spec"]["worker_sample_size"] = 1
+
             env_stepper = EnvironmentStepper(
-                environment_spec=environment_spec,
+                environment_spec=environment_spec_,
                 actor_component_spec=ActorComponent(
                     preprocessor_spec=self.preprocessing_spec,
-                    policy_spec=self.policy_spec,
+                    policy_spec=policy_spec,
                     exploration_spec=self.exploration_spec
                 ),
                 state_space=self.state_space.with_batch_rank(),
@@ -643,8 +651,7 @@ class SingleIMPALAAgent(IMPALAAgent):
 
         sub_components = [
             self.fifo_output_splitter, self.fifo_queue, self.queue_runner,
-            self.transpose_states, self.transpose_terminals, self.transpose_action_probs,
-            self.transpose_actions, self.transpose_rewards,
+            self.transposer,
             self.staging_area, self.preprocessor, self.states_dict_splitter,
             self.policy, self.loss_function, self.optimizer
         ]
@@ -653,7 +660,7 @@ class SingleIMPALAAgent(IMPALAAgent):
         self.root_component.add_components(*sub_components)
 
         # Define the Agent's (root Component's) API.
-        self.define_graph_api(*sub_components)
+        self.define_graph_api()
 
         if self.auto_build:
             self._build_graph([self.root_component], self.input_spaces, optimizer=self.optimizer,
@@ -672,30 +679,29 @@ class SingleIMPALAAgent(IMPALAAgent):
                 self.dequeue_op = self.root_component.sub_components["fifo-queue"].api_methods["get_records"]. \
                     out_op_columns[0].op_records[0].op
 
-    def define_graph_api(self, fifo_output_splitter, fifo_queue, queue_runner,
-                         transpose_states, transpose_terminals, transpose_action_probs, transpose_actions,
-                         transpose_rewards,
-                         staging_area, preprocessor, states_dict_splitter, policy, loss_function, optimizer):
+    def define_graph_api(self):
+
+        agent = self
 
         @rlgraph_api(component=self.root_component)
-        def setup_queue_runner(self_):
-            return queue_runner.setup()
+        def setup_queue_runner(root):
+            return agent.queue_runner.setup()
 
         @rlgraph_api(component=self.root_component)
-        def get_queue_size(self_):
-            return fifo_queue.get_size()
+        def get_queue_size(root):
+            return agent.fifo_queue.get_size()
 
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(self_):
+        def update_from_memory(root):
             # Pull n records from the queue.
             # Note that everything will come out as batch-major and must be transposed before the main-LSTM.
             # This is done by the network itself for all network inputs:
             # - preprocessed_s
             # - preprocessed_last_s_prime
             # But must still be done for actions, rewards, terminals here in this API-method via separate ReShapers.
-            records = fifo_queue.get_records(self.update_spec["batch_size"])
+            records = agent.fifo_queue.get_records(self.update_spec["batch_size"])
 
-            out = fifo_output_splitter.split_into_dict(records)
+            out = agent.fifo_output_splitter.split_into_dict(records)
             terminals = out["terminals"]
             states = out["states"]
             action_probs_mu = out["action_probs"]
@@ -706,56 +712,56 @@ class SingleIMPALAAgent(IMPALAAgent):
             # Flip everything to time-major.
             # TODO: Create components that are less input-space sensitive (those that have no variables should
             # TODO: be reused for any kind of processing: already done, use space_agnostic feature. See ReShape)
-            states = transpose_states.apply(states)
-            terminals = transpose_terminals.apply(terminals)
-            action_probs_mu = transpose_action_probs.apply(action_probs_mu)
+            states = agent.transposer.apply(states)
+            terminals = agent.transposer.apply(terminals)
+            action_probs_mu = agent.transposer.apply(action_probs_mu)
             actions = None
             if not self.feed_previous_action_through_nn:
-                actions = transpose_actions.apply(out["actions"])
+                actions = agent.transposer.apply(out["actions"])
             rewards = None
             if not self.feed_previous_reward_through_nn:
-                rewards = transpose_rewards.apply(out["rewards"])
+                rewards = agent.transposer.apply(out["rewards"])
 
             # If we use a GPU: Put everything on staging area (adds 1 time step policy lag, but makes copying
             # data into GPU more efficient).
             if self.has_gpu:
                 if self.has_rnn:
-                    stage_op = staging_area.stage(states, terminals, action_probs_mu, initial_internal_states)
-                    states, terminals, action_probs_mu, initial_internal_states = staging_area.unstage()
+                    stage_op = agent.staging_area.stage(states, terminals, action_probs_mu, initial_internal_states)
+                    states, terminals, action_probs_mu, initial_internal_states = agent.staging_area.unstage()
                 else:
-                    stage_op = staging_area.stage(states, terminals, action_probs_mu)
-                    states, terminals, action_probs_mu = staging_area.unstage()
+                    stage_op = agent.staging_area.stage(states, terminals, action_probs_mu)
+                    states, terminals, action_probs_mu = agent.staging_area.unstage()
             else:
                 # TODO: No-op component?
                 stage_op = None
 
             # Preprocess actions and rewards inside the state (actions: flatten one-hot, rewards: expand).
-            if self.preprocessing_required:
-                states = preprocessor.preprocess(states)
+            if agent.preprocessing_required:
+                states = agent.preprocessor.preprocess(states)
 
             # Get the pi-action probs AND the values for all our states.
-            out = policy.get_state_values_logits_probabilities_log_probs(states, initial_internal_states)
+            out = agent.policy.get_state_values_logits_probabilities_log_probs(states, initial_internal_states)
             state_values_pi = out["state_values"]
             logits_pi = out["logits"]
 
             # Isolate actions and rewards from states.
             # TODO: What if only one of actions or rewards is fed through NN, but the other not?
             if self.feed_previous_reward_through_nn and self.feed_previous_action_through_nn:
-                out = states_dict_splitter.split(states)
+                out = agent.states_dict_splitter.split(states)
                 actions = out[-2]  # TODO: Are these always the correct slots for "previous_action" and "previous_reward"?
                 rewards = out[-1]
 
             # Calculate the loss.
-            loss, loss_per_item = loss_function.loss(
+            loss, loss_per_item = agent.loss_function.loss(
                 logits_pi, action_probs_mu, state_values_pi, actions, rewards, terminals
             )
             if self.dynamic_batching:
-                policy_vars = queue_runner.data_producing_components[0].actor_component.policy._variables()
+                policy_vars = agent.queue_runner.data_producing_components[0].actor_component.policy._variables()
             else:
-                policy_vars = policy._variables()
+                policy_vars = agent.policy._variables()
 
             # Pass vars and loss values into optimizer.
-            step_op, loss, loss_per_item = optimizer.step(policy_vars, loss, loss_per_item)
+            step_op, loss, loss_per_item = agent.optimizer.step(policy_vars, loss, loss_per_item)
 
             # Return optimizer op and all loss values.
             # TODO: Make it possible to return None from API-method without messing with the meta-graph.
@@ -763,5 +769,3 @@ class SingleIMPALAAgent(IMPALAAgent):
 
     def __repr__(self):
         return "SingleIMPALAAgent()"
-
-
