@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -89,6 +89,9 @@ class Component(Specifiable):
             
             space_agnostic (bool): Whether this component does not care about input spaces (e.g. if it does not
                 create any space-dependent variables). Default: False.
+
+            nesting_level (Optional[int]): The Component's nesting level. Root component has 0.
+                None for non-placed Components. Default: None.
         """
         super(Component, self).__init__()
 
@@ -113,6 +116,7 @@ class Component(Specifiable):
         self.switched_off_apis = kwargs.pop("switched_off_apis", set())
         self.backend = kwargs.pop("backend", None)
         self.space_agnostic = kwargs.pop("space_agnostic", None)
+        self.nesting_level = kwargs.pop("nesting_level", None)
 
         assert not kwargs, "ERROR: kwargs ({}) still contains items!".format(kwargs)
 
@@ -517,10 +521,10 @@ class Component(Specifiable):
             # Numpyize initializer and give it correct dtype.
             else:
                 shape = None
-                initializer = np.asarray(initializer, dtype=util.dtype(dtype, "np"))
+                initializer = np.asarray(initializer, dtype=util.convert_dtype(dtype, "np"))
 
             var = tf.get_variable(
-                name=name, shape=shape, dtype=util.dtype(dtype), initializer=initializer, trainable=trainable,
+                name=name, shape=shape, dtype=util.convert_dtype(dtype), initializer=initializer, trainable=trainable,
                 collections=[tf.GraphKeys.GLOBAL_VARIABLES if local is False else tf.GraphKeys.LOCAL_VARIABLES],
                 use_resource=use_resource
             )
@@ -531,7 +535,7 @@ class Component(Specifiable):
             )
 
             var = eager.Variable(
-                name=name, shape=shape, dtype=util.dtype(dtype), initializer=initializer, trainable=trainable,
+                name=name, shape=shape, dtype=util.convert_dtype(dtype), initializer=initializer, trainable=trainable,
                 collections=[tf.GraphKeys.GLOBAL_VARIABLES if local is False else tf.GraphKeys.LOCAL_VARIABLES]
             )
 
@@ -704,7 +708,7 @@ class Component(Specifiable):
                     if get_ref:
                         variables[re.sub(r'/', custom_scope_separator, name)] = self.variables[name]
                     else:
-                        variables[re.sub(r'/', custom_scope_separator, name)] = self.variables[name].get_value()
+                        variables[re.sub(r'/', custom_scope_separator, name)] = self.read_variable(self.variables[name])
                 elif global_scope_name in self.variables:
                     if global_scope:
                         if get_ref:
@@ -712,15 +716,15 @@ class Component(Specifiable):
                                 self.variables[global_scope_name]
                         else:
                             variables[re.sub(r'/', custom_scope_separator, global_scope_name)] = \
-                                self.variables[global_scope_name].get_value()
+                                self.read_variable(self.variables[global_scope_name])
                     else:
                         if get_ref:
                             variables[name] = self.variables[global_scope_name]
                         else:
-                            variables[name] = self.variables[global_scope_name].get_value()
+                            variables[name] = self.read_variable(self.variables[global_scope_name])
         return variables
 
-    def create_summary(self, name, values, type_="histogram"):
+    def create_summary(self, name, values, summary_type="histogram"):
         """
         Creates a summary op (and adds it to the graph).
         Skips those, whose full name does not match `self.summary_regexp`.
@@ -732,7 +736,7 @@ class Component(Specifiable):
 
             values (op): The op to summarize.
 
-            type\_ (str): The summary type to create. Currently supported are:
+            summary_type (str): The summary type to create. Currently supported are:
                 "histogram", "scalar" and "text".
         """
         # Prepend the "summaries/"-prefix.
@@ -745,35 +749,35 @@ class Component(Specifiable):
 
         summary = None
         if get_backend() == "tf":
-            ctor = getattr(tf.summary, type_)
+            ctor = getattr(tf.summary, summary_type)
             summary = ctor(name, values)
 
         # Registers the new summary with this Component.
         if global_name in self.summaries:
             raise RLGraphError("ERROR: Summary with name '{}' already exists in {}'s summary "
-                            "registry!".format(global_name, self.name))
+                               "registry!".format(global_name, self.name))
         self.summaries[global_name] = summary
         self.propagate_summary(global_name)
 
-    def propagate_summary(self, key_):
+    def propagate_summary(self, summary_key):
         """
         Propagates a single summary op of this Component to its parents' summaries registries.
 
         Args:
-            key\_ (str): The lookup key for the summary to propagate.
+            summary_key (str): The lookup key for the summary to propagate.
         """
         # Return if there is no parent.
         if self.parent_component is None:
             return
 
         # If already there -> Error.
-        if key_ in self.parent_component.summaries:
+        if summary_key in self.parent_component.summaries:
             raise RLGraphError("ERROR: Summary registry of '{}' already has a summary under key '{}'!".
-                format(self.parent_component.name, key_))
-        self.parent_component.summaries[key_] = self.summaries[key_]
+                               format(self.parent_component.name, summary_key))
+        self.parent_component.summaries[summary_key] = self.summaries[summary_key]
 
         # Recurse up the container hierarchy.
-        self.parent_component.propagate_summary(key_)
+        self.parent_component.propagate_summary(summary_key)
 
     def add_components(self, *components, **kwargs):
         """
@@ -807,12 +811,15 @@ class Component(Specifiable):
                                    format(component.name))
             # Make sure each Component can only be added once to a parent/container Component.
             elif component.parent_component is not None:
-                raise RLGraphError("ERROR: Sub-Component with name '{}' has already been added once to a container "
-                                "Component! Each Component can only be added once to a parent.".format(component.name))
+                raise RLGraphError(
+                    "ERROR: Sub-Component with name '{}' has already been added once to a container Component! Each "
+                    "Component can only be added once to a parent.".format(component.name)
+                )
             # Make sure we don't add to ourselves.
             elif component is self:
                 raise RLGraphError("ERROR: Cannot add a Component ({}) as a sub-Component to itself!".format(self.name))
             component.parent_component = self
+            component.nesting_level = (self.nesting_level or 0) + 1
             self.sub_components[component.name] = component
 
             # Fix the sub-component's (and sub-sub-component's etc..) scope(s).
@@ -820,7 +827,8 @@ class Component(Specifiable):
 
             # Execution modes must be coherent within one component subgraph.
             self.propagate_sub_component_properties(
-                properties=dict(execution_mode=self.execution_mode), component=component
+                properties=dict(execution_mode=self.execution_mode, nesting_level=(self.nesting_level or 0) + 1),
+                component=component
             )
 
             # Should we expose some API-methods of the child?
@@ -832,10 +840,10 @@ class Component(Specifiable):
                     must_be_complete = api_method_rec.must_be_complete
 
                     @rlgraph_api(component=self, name=api_method_name, must_be_complete=must_be_complete)
-                    def exposed_api_method_wrapper(self, *inputs):
+                    def exposed_api_method_wrapper(self, *inputs, **kwargs):
                         # Complicated way to lookup sub-component's method to avoid fixtures when original
                         # component gets copied.
-                        return getattr(self.sub_components[component_name], name_)(*inputs)
+                        return getattr(self.sub_components[component_name], name_)(*inputs, **kwargs)
 
         # Add own reusable scope to front of all sub-components' reusable scope.
         if self.reuse_variable_scope is not None:
@@ -941,7 +949,7 @@ class Component(Specifiable):
         Returns:
             List[Component]: A list (may be empty if this component has no parents) of all parent and grand-parents.
         """
-        ret = list()
+        ret = []
         component = self
         while component.parent_component is not None:
             ret.append(component.parent_component)
@@ -995,6 +1003,9 @@ class Component(Specifiable):
             # Normal property: Set to static given value.
             else:
                 setattr(component, name, value)
+                # Nesting_level: Increase by one the deeper we go.
+                if name == "nesting_level":
+                    properties_scoped[name] = value + 1
         for sc in component.sub_components.values():
             component.propagate_sub_component_properties(properties_scoped, sc, recursive_=True)
 
@@ -1148,21 +1159,24 @@ class Component(Specifiable):
             # Lists or numpy arrays may be used to store mutable state that does not need
             # tensor operations.
             elif isinstance(variable, list) or isinstance(variable, np.ndarray):
-                ret = []
-                for i in indices:
-                    val = variable[i]
-                    # Type checking is necessary because torch.stack only works on same types.
-                    if isinstance(val, torch.Tensor):
-                        if dtype is None:
-                            ret.append(val)
-                        elif dtype == torch.float32:
-                            ret.append(val.float())
-                        elif dtype == torch.int32:
-                            ret.append(val.int())
-                        elif dtype == torch.uint8:
-                            ret.append(val.byte())
-                # Stack list into one Tensor with a btach dim.
-                return torch.stack(ret)
+                if indices is not None:
+                    ret = []
+                    for i in indices:
+                        val = variable[i]
+                        # Type checking is necessary because torch.stack only works on same types.
+                        if isinstance(val, torch.Tensor):
+                            if dtype is None:
+                                ret.append(val)
+                            elif dtype == torch.float32:
+                                ret.append(val.float())
+                            elif dtype == torch.int32:
+                                ret.append(val.int())
+                            elif dtype == torch.uint8:
+                                ret.append(val.byte())
+                    # Stack list into one Tensor with a btach dim.
+                    return torch.stack(ret)
+                else:
+                    return variable
 
     def sub_component_by_name(self, scope_name):
         """
