@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ from rlgraph.components.explorations.epsilon_exploration import EpsilonExplorati
 from rlgraph.components.common.noise_components import NoiseComponent
 from rlgraph.spaces import IntBox, FloatBox
 from rlgraph.spaces.space_utils import sanity_check_space
-from rlgraph.utils.util import dtype
+from rlgraph.utils.util import convert_dtype
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
 
 if get_backend() == "tf":
@@ -133,15 +133,15 @@ class Exploration(Component):
             DataOp: The DataOp representing the action. This will match the shape of self.action_space.
         """
         if get_backend() == "tf":
-            random_actions = tf.random_uniform(
-                shape=tf.shape(sample),
-                maxval=self.flat_action_space[key].num_categories,
-                dtype=dtype("int")
-            )
-
             if use_exploration is False:
                 return sample
             else:
+                random_actions = tf.random_uniform(
+                    shape=tf.shape(sample),
+                    maxval=self.flat_action_space[key].num_categories,
+                    dtype=convert_dtype("int")
+                )
+
                 return tf.where(
                     # `use_exploration` given as actual bool or as tensor?
                     condition=epsilon_decisions if use_exploration is True else tf.logical_and(
