@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -306,6 +306,7 @@ class SequenceHelper(Component):
             Sequence of deltas.
         """
         if get_backend() == "tf":
+            values = tf.squeeze(input=values)
             num_values = tf.shape(input=values)[0]
 
             # Again ensure last index is 1 for any sub-sample arriving here.
@@ -374,6 +375,7 @@ class SequenceHelper(Component):
             return tf.squeeze(deltas)
         elif get_backend() == "pytorch":
             deltas = []
+            discount_tensor = torch.tensor(discount)
             start_index = 0
             i = 0
             if len(values) > 1:
@@ -392,7 +394,7 @@ class SequenceHelper(Component):
                     adjusted_v = torch.tensor(baseline_slice)
 
                     # +1 because we want to include i-th value.
-                    delta = rewards[start_index:i + 1] + discount * adjusted_v[1:] - adjusted_v[:-1]
+                    delta = rewards[start_index:i + 1] + discount_tensor * adjusted_v[1:] - adjusted_v[:-1]
                     deltas.extend(delta)
                     start_index = i + 1
                 i += 1
