@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -40,7 +40,7 @@ class LSTMLayer(NNLayer):
     """
     def __init__(
             self, units, use_peepholes=False, cell_clip=None, static_loop=False,
-            forget_bias=1.0, parallel_iterations=32,
+            forget_bias=1.0, parallel_iterations=32, return_sequences=True,
             swap_memory=False, time_major=False, **kwargs):  # weights_spec=None, dtype="float"
         """
         Args:
@@ -58,6 +58,8 @@ class LSTMLayer(NNLayer):
             forget_bias (float): The forget gate bias to use. Default: 1.0.
             parallel_iterations (int): The number of iterations to run in parallel.
                 Default: 32.
+            return_sequences (bool): Whether to return one output for each input or only the last output.
+                Default: True.
             swap_memory (bool): Transparently swap the tensors produced in forward inference but needed for back
                 prop from GPU to CPU. This allows training RNNs which would typically not fit on a single GPU,
                 with very minimal (or no) performance penalty.
@@ -82,6 +84,7 @@ class LSTMLayer(NNLayer):
         self.forget_bias = forget_bias
 
         self.parallel_iterations = parallel_iterations
+        self.return_sequences = return_sequences
         self.swap_memory = swap_memory
         self.in_space = None
 
@@ -152,10 +155,12 @@ class LSTMLayer(NNLayer):
         Args:
             inputs (SingleDataOp): The data to pass through the layer (batch of n items, m timesteps).
                 Position of batch- and time-ranks in the input depend on `self.time_major` setting.
+
             initial_c_and_h_states (DataOpTuple): The initial cell- and hidden-states to use.
-                None for the default behavior (TODO: describe here what default means: zero?)
+                None for the default behavior (all zeros).
                 The cell-state in an LSTM is passed between cells from step to step and only affected by element-wise
                 operations. The hidden state is identical to the output of the LSTM on the previous time step.
+
             sequence_length (Optional[SingleDataOp]): An int tensor mapping each batch item to a sequence length
                 such that the remaining time slots for each batch item are filled with zeros.
 
@@ -185,6 +190,13 @@ class LSTMLayer(NNLayer):
                 )
             # We are running with a fixed number of time steps (static unroll).
             else:
+                # Set to zeros as tf lstm object does not handle None.
+                if initial_c_and_h_states is None:
+                    shape = (tf.shape(inputs)[0 if self.in_space.time_major is False else 1], self.units)
+                    initial_c_and_h_states = tf.nn.rnn_cell.LSTMStateTuple(
+                        tf.zeros(shape=shape, dtype=tf.float32),
+                        tf.zeros(shape=shape, dtype=tf.float32)
+                    )
                 output_list = list()
                 lstm_state_tuple = initial_c_and_h_states
                 # TODO: Add option to reset the internal state in the middle of this loop iff some reset signal
@@ -199,13 +211,24 @@ class LSTMLayer(NNLayer):
                     output_list.append(output)
                 lstm_out = tf.stack(output_list)
 
+            # Only return last value.
+            if self.return_sequences is False:
+                if self.in_space.time_major is True:
+                    lstm_out = lstm_out[-1]
+                else:
+                    lstm_out = lstm_out[:,-1]
+                lstm_out._batch_rank = 0
+            # Return entire sequence.
+            else:
+                lstm_out._batch_rank = 0 if self.in_space.time_major is False else 1
+                lstm_out._time_rank = 0 if self.in_space.time_major is True else 1
+
             # Returns: Unrolled-outputs (time series of all encountered h-states), final c- and h-states.
-            lstm_out._batch_rank = 0 if self.in_space.time_major is False else 1
-            lstm_out._time_rank = 0 if self.in_space.time_major is True else 1
             return lstm_out, DataOpTuple(lstm_state_tuple)
 
         elif get_backend() == "pytorch":
             # TODO init hidden state has to be available at create variable time to use.
             inputs = torch.cat(inputs).view(len(inputs), 1, -1)
+            # TODO: support `self.return_sequences` = False
             out, self.hidden_state = self.lstm(inputs, self.hidden_state)
             return out, DataOpTuple(self.hidden_state)
