@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -139,15 +139,6 @@ class ReShape(PreprocessLayer):
             num_categories = self.flatten_categories
         return num_categories
 
-    def check_input_spaces(self, input_spaces, action_space=None):
-        super(ReShape, self).check_input_spaces(input_spaces, action_space)
-
-        # Check whether our input space has-batch or not and store this information here.
-        in_space = input_spaces["preprocessing_inputs"]  # type: Space
-
-        if self.flatten is True and isinstance(in_space, IntBox) and self.flatten_categories is True:
-            sanity_check_space(in_space, must_have_categories=True, num_categories=(2, 10000))
-
     @rlgraph_api(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
     def _graph_fn_apply(self, key, preprocessing_inputs, input_before_time_rank_folding=None):
         """
@@ -244,11 +235,16 @@ class ReShape(PreprocessLayer):
                 )
                 if hasattr(preprocessing_inputs, "_batch_rank"):
                     preprocessing_inputs_._batch_rank = preprocessing_inputs._batch_rank
+                if hasattr(preprocessing_inputs, "_time_rank"):
                     preprocessing_inputs_._time_rank = preprocessing_inputs._time_rank
                 preprocessing_inputs = preprocessing_inputs_
 
             if self.unfold_time_rank:
-                new_shape = (-1, -1) + tuple(preprocessing_inputs.shape.as_list()[1:])
+                list_shape = preprocessing_inputs.shape.as_list()
+                assert len(list_shape) == 1 or list_shape[1] is not None,\
+                    "ERROR: Cannot unfold. `preprocessing_inputs` (with shape {}) " \
+                    "already seems to be unfolded!".format(list_shape)
+                new_shape = (-1, -1) + tuple(list_shape[1:])
             elif self.fold_time_rank:
                 new_shape = (-1,) + tuple(preprocessing_inputs.shape.as_list()[2:])
             else:
@@ -281,7 +277,7 @@ class ReShape(PreprocessLayer):
                 if self.fold_time_rank:
                     reshaped._batch_rank = 0
                 elif self.unfold_time_rank:
-                    reshaped._batch_rank = 0 if self.time_major is False else 1
+                    reshaped._batch_rank = 1 if self.time_major is True else 0
                     reshaped._time_rank = 0 if self.time_major is True else 1
                 else:
                     if space.has_batch_rank is True:
