@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,6 +41,7 @@ class DQFDLossFunction(DQNLossFunction):
         Args:
             expert_margin (float): The expert margin enforces a distance in Q-values between expert action and
                 all other actions.
+
             supervised_weight (float): Indicates weight of the expert loss.
         """
         self.expert_margin = expert_margin
@@ -59,11 +60,11 @@ class DQFDLossFunction(DQNLossFunction):
     @rlgraph_api
     def loss_per_item(self, q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp=None,
                       importance_weights=None, apply_demo_loss=False):
-
-        # Get the targets -> inherited from DQNLossFunction
+        # Get the targets per action.
         td_targets = self._graph_fn_get_td_targets(rewards, terminals, qt_values_sp, q_values_sp)
         # Average over container sub-actions.
-        td_targets = self._graph_fn_average_over_container_keys(td_targets)
+        if self.shared_container_action_target is True:
+            td_targets = self._graph_fn_average_over_container_keys(td_targets)
 
         # Calculate the loss per item.
         loss_per_item = self._graph_fn_loss_per_item(td_targets, q_values_s, actions,
@@ -71,6 +72,9 @@ class DQFDLossFunction(DQNLossFunction):
         # Average over container sub-actions.
         loss_per_item = self._graph_fn_average_over_container_keys(loss_per_item)
 
+        # Apply huber loss.
+        loss_per_item = self._graph_fn_apply_huber_loss_if_necessary(loss_per_item)
+
         return loss_per_item
 
     @graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
@@ -119,8 +123,9 @@ class DQFDLossFunction(DQNLossFunction):
 
             # Apply importance-weights from a prioritized replay to the loss.
             if self.importance_weights:
-                return importance_weights * self._apply_huber_loss_if_necessary(td_delta)
+                return importance_weights * td_delta
             else:
-                return self._apply_huber_loss_if_necessary(td_delta)
+                return td_delta
+
 
 
