@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,6 @@ from rlgraph.spaces import IntBox, ContainerSpace
 from rlgraph.spaces.space_utils import sanity_check_space
 from rlgraph.utils import pytorch_one_hot
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
-from rlgraph.utils.pytorch_util import pytorch_reduce_mean
 from rlgraph.utils.util import get_rank
 
 if get_backend() == "tf":
@@ -47,13 +46,16 @@ class DQNLossFunction(LossFunction):
     [4] Action Branching Architectures for Deep Reinforcement Learning. Tavakoli, Pardo, and Kormushev - 2017
     """
     def __init__(self, double_q=False, huber_loss=False, importance_weights=False, n_step=1,
-                 scope="dqn-loss-function", **kwargs):
+                 shared_container_action_target=True, scope="dqn-loss-function", **kwargs):
         """
         Args:
             double_q (bool): Whether to use the double DQN loss function ([2]).
             huber_loss (bool): Whether to apply a huber loss correction ([3]).
             importance_weights (bool): Where to use importance weights from a prioritized replay.
             n_step (int): n-step adjustment to discounting.
+
+            shared_container_action_target (bool): Whether - only in the case of container actions - the target term
+                should be shared (average) over all action components' single loss terms. Default: True.
         """
         self.double_q = double_q
         self.huber_loss = huber_loss
@@ -61,6 +63,7 @@ class DQNLossFunction(LossFunction):
         # TODO reward must be preprocessed to work correctly for n-step.
         # For Apex, this is done in the worker - do we want to move this as an in-graph option too?
         self.n_step = n_step
+        self.shared_container_action_target = shared_container_action_target
 
         # Clip value, see: https://en.wikipedia.org/wiki/Huber_loss
         self.huber_delta = kwargs.get("huber_delta", 1.0)
@@ -94,16 +97,20 @@ class DQNLossFunction(LossFunction):
     @rlgraph_api
     def loss_per_item(self, q_values_s, actions, rewards, terminals, qt_values_sp, q_values_sp=None,
                       importance_weights=None):
-        # Get the targets.
+        # Get the targets per action.
         td_targets = self._graph_fn_get_td_targets(rewards, terminals, qt_values_sp, q_values_sp)
         # Average over container sub-actions.
-        td_targets = self._graph_fn_average_over_container_keys(td_targets)
+        if self.shared_container_action_target is True:
+            td_targets = self._graph_fn_average_over_container_keys(td_targets)
 
         # Calculate the loss per item.
         loss_per_item = self._graph_fn_loss_per_item(td_targets, q_values_s, actions, importance_weights)
         # Average over container sub-actions.
         loss_per_item = self._graph_fn_average_over_container_keys(loss_per_item)
 
+        # Apply huber loss.
+        loss_per_item = self._graph_fn_apply_huber_loss_if_necessary(loss_per_item)
+
         return loss_per_item
 
     @graph_fn(flatten_ops=True, split_ops=True, add_auto_key_as_first_param=True)
@@ -209,9 +216,12 @@ class DQNLossFunction(LossFunction):
         Args:
             td_targets (SingleDataOp): The already calculated TD-target terms (r + gamma maxa'Qt(s',a')
                 OR for double Q: r + gamma Qt(s',argmaxa'(Q(s',a'))))
+
             q_values_s (SingleDataOp): The batch of Q-values representing the expected accumulated discounted returns
                 when in s and taking different actions a.
+
             actions (SingleDataOp): The batch of actions that were actually taken in states s (from a memory).
+
             importance_weights (Optional[SingleDataOp]): If 'self.importance_weights' is True: The batch of weights to
                 apply to the losses.
 
@@ -266,15 +276,16 @@ class DQNLossFunction(LossFunction):
 
             # Reduce over the composite actions, if any.
             if get_rank(td_delta) > 1:
-                td_delta = pytorch_reduce_mean(td_delta, list(range(1, self.ranks_to_reduce + 1)), keepdims=False)
+                td_delta = torch.mean(td_delta, tuple(range(1, self.ranks_to_reduce + 1)), keepdim=False)
 
         # Apply importance-weights from a prioritized replay to the loss.
         if self.importance_weights:
-            return importance_weights * self._apply_huber_loss_if_necessary(td_delta)
+            return importance_weights * td_delta
         else:
-            return self._apply_huber_loss_if_necessary(td_delta)
+            return td_delta
 
-    def _apply_huber_loss_if_necessary(self, td_delta):
+    @graph_fn
+    def _graph_fn_apply_huber_loss_if_necessary(self, td_delta):
         if self.backend == "python" or get_backend() == "python":
             if self.huber_loss:
                 return np.where(
