@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -93,13 +93,11 @@ class QueueRunner(Component):
 
                 # TODO: specific for IMPALA problem: needs to be generalized.
                 if self.internal_states_slicer is not None:
-                    terminals, states, actions, rewards, action_log_probs, internal_states = \
-                        self.env_output_splitter.split(record)
+                    outs = self.env_output_splitter.split(record)
 
-                    initial_internal_states = self.internal_states_slicer.slice(internal_states, 0)
-                    record = self.fifo_input_merger.merge(
-                        terminals, states, actions, rewards, action_log_probs, initial_internal_states
-                    )
+                    # Assume that internal_states are the last item coming from the env-stepper.
+                    initial_internal_states = self.internal_states_slicer.slice(outs[-1], 0)
+                    record = self.fifo_input_merger.merge(*(outs[:-1] + (initial_internal_states,)))
                 else:
                     terminals, states, actions, rewards, action_log_probs = self.env_output_splitter.split(record)
                     record = self.fifo_input_merger.merge(
