@@ -17,7 +17,6 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from rlgraph.components.action_adapters.baseline_action_adapter import BaselineActionAdapter
 from rlgraph.components.component import Component
 from rlgraph.components.neural_networks.preprocessor_stack import PreprocessorStack
 from rlgraph.components.policies.policy import Policy
@@ -30,7 +29,7 @@ class ActorComponent(Component):
     A Component that incorporates an entire pipeline from env state to an action choice.
     Includes preprocessor, policy and exploration sub-components.
     """
-    def __init__(self, preprocessor_spec, policy_spec, exploration_spec, #deterministic_policy=None,
+    def __init__(self, preprocessor_spec, policy_spec, exploration_spec=None,
                  **kwargs):
         """
         Args:
