@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ import copy
 from rlgraph import get_backend
 from rlgraph.utils.decorators import rlgraph_api
 from rlgraph.components.component import Component
+from rlgraph.components.layers.preprocessing import ReShape
 from rlgraph.utils.util import force_tuple, force_list
 
 
@@ -41,18 +42,25 @@ class Stack(Component):
                 to each other.
 
         Keyword Args:
-            api_methods (Set[Union[str,Tuple[str,str]]]): A set of names of API-methods to connect through the stack.
+            api_methods (List[Union[str,Tuple[str,str],dict]]): A list of strings of API-methods names to connect
+                through the stack.
                 Defaults to {"apply"}. All sub-Components must implement all API-methods in this set.
-                Alternatively, a tuple can be used  (instead of a string), in which case the first tuple-item
-                is used as the Stack's API-method name and the second item is the sub-Components' API-method name.
+                Alternatively, this set may contain tuples (1st item is the final Stack's API method name, 2nd item
+                is the name of the API-methods of the sub-Components to connect through).
                 E.g. api_methods={("stack_run", "run")}. This will create "stack_run" for the Stack, which will call
                 - one by one - all the "run" methods of the sub-Components.
+                Alternatively, this set may contain spec-dicts with keys:
+                `api` (exposed final API-method name), `component_api` (sub-Components API-method names to connect
+                through), `function` (the custom API-function to use), `fold_time_rank` (whether to fold a time
+                rank into a batch rank at the beginning), `unfold_time_rank` (whether to unfold the time rank
+                at the end).
 
                 Connecting always works by first calling the first sub-Component's API-method, then - with the
                 result - calling the second sub-Component's API-method, etc..
-                This is done for all API-methods in the given set.
+                This is done for all API-methods in the given set, plus - optionally - time rank folding and unfolding
+                at the beginning and/or end.
         """
-        api_methods = kwargs.pop("api_methods", {"apply"})
+        self.api_methods_options = kwargs.pop("api_methods", ["apply"])
         super(Stack, self).__init__(*sub_components, scope=kwargs.pop("scope", "stack"), **kwargs)
 
         self.num_allowed_inputs = None
@@ -60,7 +68,12 @@ class Stack(Component):
 
         self.map_api_to_sub_components_api = dict()
 
-        self._build_stack(api_methods)
+        self.folder = ReShape(fold_time_rank=True, scope="time-rank-folder_")
+        self.unfolder = ReShape(unfold_time_rank=True, scope="time-rank-unfolder_")
+
+        self.add_components(self.folder, self.unfolder)
+
+        self._build_stack(self.api_methods_options)
 
     def _build_stack(self, api_methods):
         """
@@ -69,14 +82,23 @@ class Stack(Component):
         by that name.
 
         Args:
-            api_methods (Set[Union[str,Tuple[str,str]]]): See ctor kwargs.
+            api_methods (List[Union[str,Tuple[str,str],dict]]): See ctor kwargs.
         """
         # Loop through the API-method set and register each one.
         for api_method_spec in api_methods:
             function_to_use = None
+            fold_time_rank = False
+            unfold_time_rank = False
 
+            # Detailed spec-dict given.
+            if isinstance(api_method_spec, dict):
+                stack_api_method_name = api_method_spec["api"]
+                component_api_method_name = api_method_spec.get("component_api", stack_api_method_name)
+                function_to_use = api_method_spec.get("function")
+                fold_time_rank = api_method_spec.get("fold_time_rank", False)
+                unfold_time_rank = api_method_spec.get("unfold_time_rank", False)
             # API-method of sub-Components and this Stack should have different names.
-            if isinstance(api_method_spec, tuple):
+            elif isinstance(api_method_spec, tuple):
                 # Custom method given, use that instead of creating one automatically.
                 if callable(api_method_spec[1]):
                     stack_api_method_name = component_api_method_name = api_method_spec[0]
@@ -91,16 +113,18 @@ class Stack(Component):
 
             # API-method for this Stack does not exist yet -> Automatically create it.
             if not hasattr(self, stack_api_method_name):
-
                 # Custom API-method is given (w/o decorator) -> Call the decorator directly here to register it.
                 if function_to_use is not None:
                     rlgraph_api(api_method=function_to_use, component=self, name=stack_api_method_name)
-
                 # No API-method given -> Create auto-API-method and set it up through decorator.
                 else:
-                    self.build_auto_api_method(stack_api_method_name, component_api_method_name)
+                    self.build_auto_api_method(
+                        stack_api_method_name, component_api_method_name, fold_time_rank=fold_time_rank,
+                        unfold_time_rank=unfold_time_rank
+                    )
 
-    def build_auto_api_method(self, stack_api_method_name, sub_components_api_method_name, ok_to_overwrite=False):
+    def build_auto_api_method(self, stack_api_method_name, sub_components_api_method_name,
+                              fold_time_rank=False, unfold_time_rank=False, ok_to_overwrite=False):
         """
         Creates and registers an auto-API method for this stack.
 
@@ -114,9 +138,25 @@ class Stack(Component):
         """
         @rlgraph_api(name=stack_api_method_name, component=self, ok_to_overwrite=ok_to_overwrite)
         def method(self_, *inputs, **kwargs):
-            args_ = inputs
+            # Fold time rank? For now only support 1st arg folding/unfolding.
+            original_input = inputs[0]
+            if fold_time_rank is True:
+                args_ = tuple([self.folder.apply(original_input)] + list(inputs[1:]))
+            else:
+                # TODO: If only unfolding: Assume for now that 2nd input is the original one (so we can infer
+                # TODO: batch/time dims).
+                if unfold_time_rank is True:
+                    assert len(inputs) >= 2, \
+                        "ERROR: In Stack: If unfolding w/o folding, second arg must be the original input!"
+                    original_input = inputs[1]
+                    args_ = tuple([inputs[0]] + list(inputs[2:]))
+                else:
+                    args_ = inputs
             kwargs_ = kwargs
+
             for i, sub_component in enumerate(self_.sub_components.values()):  # type: Component
+                if sub_component.scope in ["time-rank-folder_", "time-rank-unfolder_"]:
+                    continue
                 # TODO: python-Components: For now, we call each preprocessor's graph_fn
                 #  directly (assuming that inputs are not ContainerSpaces).
                 if self_.backend == "python" or get_backend() == "python":
@@ -140,11 +180,24 @@ class Stack(Component):
                     kwargs_ = {}
 
             if args_ == ():
+                # Unfold time rank? For now only support 1st arg folding/unfolding.
+                if unfold_time_rank is True:
+                    assert len(kwargs_) == 1,\
+                        "ERROR: time-rank-unfolding not supported for more than one NN-return value!"
+                    key = next(iter(kwargs_))
+                    kwargs_ = {key: self.unfolder.apply(kwargs_[key], original_input)}
                 return kwargs_
-            elif len(args_) == 1:
-                return args_[0]
             else:
-                return args_
+                # Unfold time rank? For now only support 1st arg folding/unfolding.
+                if unfold_time_rank is True:
+                    assert len(args_) == 1,\
+                        "ERROR: time-rank-unfolding not supported for more than one NN-return value!"
+                    args_ = tuple([self.unfolder.apply(args_[0], original_input)] +
+                                  list(args_[1 if fold_time_rank is True else 2:]))
+                if len(args_) == 1:
+                    return args_[0]
+                else:
+                    return args_
 
     @classmethod
     def from_spec(cls, spec=None, **kwargs):
