@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -35,8 +35,8 @@ from rlgraph.spaces.space_utils import get_space_from_op, check_space_equivalenc
 from rlgraph.utils.input_parsing import parse_summary_spec
 from rlgraph.utils.util import force_list, force_tuple, get_shape
 from rlgraph.utils.ops import is_constant, ContainerDataOp, flatten_op
-from rlgraph.utils.op_records import FlattenedDataOp, DataOpRecord, DataOpRecordColumnIntoGraphFn, \
-    DataOpRecordColumnIntoAPIMethod, DataOpRecordColumnFromGraphFn, get_call_param_name
+from rlgraph.utils.op_records import FlattenedDataOp, DataOpRecord, DataOpRecordColumn, DataOpRecordColumnIntoGraphFn, \
+    DataOpRecordColumnIntoAPIMethod, DataOpRecordColumnFromGraphFn, DataOpRecordColumnFromAPIMethod, get_call_param_name
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -92,12 +92,14 @@ class GraphBuilder(Specifiable):
         self.num_ops = 0
         # Number of trainable variables (optimizable weights).
         self.num_trainable_parameters = 0
+        self.graph_call_times = []
+        self.var_call_times = []
 
         # Create an empty root-Component into which everything will be assembled by an Algo.
         self.root_component = None
 
         # Maps API method names to in- (placeholders) and out op columns (ops to pull).
-        self.api = dict()
+        self.api = {}
 
         self.op_records_to_process = set()
         self.op_records_to_process_later = set()
@@ -199,7 +201,7 @@ class GraphBuilder(Specifiable):
             self.op_records_to_process.update(component.constant_op_records)
             self.build_component_when_input_complete(component)
 
-        op_records_list = sorted(self.op_records_to_process, key=lambda rec: rec.id)
+        op_records_list = self._sort_op_recs(self.op_records_to_process)
 
         # Re-iterate until our bag of op-recs to process is empty.
         iterations = self._build(op_records_list)
@@ -331,25 +333,38 @@ class GraphBuilder(Specifiable):
                         # Keep working with the generated output ops.
                         self.op_records_to_process.update(no_in_col.out_graph_fn_column.op_records)
 
-                # Now that the component is input-complete, the parent may have become variable-complete.
-                if component.parent_component is not None:
-                    self.build_component_when_input_complete(component.parent_component)
+                # Now that the component is built, the parent may have become variable-complete.
+                #if component.parent_component is not None:
+                #    self.build_component_when_input_complete(component.parent_component)
 
         # Check variable-completeness and actually call the _variable graph_fn if not already done so.
+        # Collect sub-components and build them as well if they just became variable-complete.
+        sub_components = component.sub_components.values()
+        sub_components_not_var_complete = set()
+        for sub_component in sub_components:
+            if sub_component.variable_complete is False:
+                sub_components_not_var_complete.add(sub_component)
+
         if component.input_complete is True and component.check_variable_completeness():
-            # The graph_fn _variables has some in-op-columns that need to be run through the function.
-            if "_graph_fn__variables" in component.graph_fns:
-                graph_fn_rec = component.graph_fns["_graph_fn__variables"]
-                # TODO: Think about only running through no-input-graph-fn once, no matter how many in-op-columns it has.
-                # TODO: Then link the first in-op-column (empty) to all out-op-columns.
-                for i, in_op_col in enumerate(graph_fn_rec.in_op_columns):
-                    if in_op_col.already_sent is False:
-                        self.run_through_graph_fn_with_device_and_scope(in_op_col)
-                        # If graph_fn_rec doesn't know about the out-op-col yet, add it.
-                        if len(graph_fn_rec.out_op_columns) <= i:
-                            assert len(graph_fn_rec.out_op_columns) == i  # make sure, it's really just one col missing
-                            graph_fn_rec.out_op_columns.append(in_op_col.out_graph_fn_column)
-                        self.op_records_to_process.update(graph_fn_rec.out_op_columns[i].op_records)
+                # The graph_fn _variables has some in-op-columns that need to be run through the function.
+                if "_graph_fn__variables" in component.graph_fns:
+                    graph_fn_rec = component.graph_fns["_graph_fn__variables"]
+                    # TODO: Think about only running through no-input-graph-fn once, no matter how many in-op-columns it has.
+                    # TODO: Then link the first in-op-column (empty) to all out-op-columns.
+                    for i, in_op_col in enumerate(graph_fn_rec.in_op_columns):
+                        if in_op_col.already_sent is False:
+                            self.run_through_graph_fn_with_device_and_scope(in_op_col)
+                            # If graph_fn_rec doesn't know about the out-op-col yet, add it.
+                            if len(graph_fn_rec.out_op_columns) <= i:
+                                assert len(graph_fn_rec.out_op_columns) == i  # make sure, it's really just one col missing
+                                graph_fn_rec.out_op_columns.append(in_op_col.out_graph_fn_column)
+                            self.op_records_to_process.update(graph_fn_rec.out_op_columns[i].op_records)
+
+                for sub_component in sub_components_not_var_complete:
+                    self.build_component_when_input_complete(sub_component)
+                # Now that the component is variable-complete, the parent may have become variable-complete as well.
+                #if component.parent_component is not None:
+                #    self.build_component_when_input_complete(component.parent_component)
 
     def run_through_graph_fn_with_device_and_scope(self, op_rec_column, create_new_out_column=None):
         """
@@ -364,6 +379,12 @@ class GraphBuilder(Specifiable):
                 an error.
                 Default: False.
         """
+        if op_rec_column.already_sent is not False:
+            raise RLGraphBuildError(
+                "op_rec_column ID={} already sent through graph_fn '{}'! Cannot do so again.".format(
+                    op_rec_column.id, op_rec_column.graph_fn.__name__)
+            )
+
         # Get the device for the ops generated in the graph_fn (None for custom device-definitions within the graph_fn).
         device = self.get_device(op_rec_column.component, variables=False)
 
@@ -407,7 +428,7 @@ class GraphBuilder(Specifiable):
             op_rec_column.out_graph_fn_column = out_op_rec_column
 
         # Tag column as already sent through graph_fn.
-        op_rec_column.already_sent = True  # TODO: assert is False before this?
+        op_rec_column.already_sent = True
         return op_rec_column.out_graph_fn_column
 
     def get_device(self, component, variables=False):
@@ -469,7 +490,7 @@ class GraphBuilder(Specifiable):
         for api_method in api_methods:
             # Start with the input set for this method.
             api_input_records = self.root_component.api_methods[api_method].in_op_columns[0].op_records
-            op_records_list = sorted(api_input_records, key=lambda rec: rec.id)
+            op_records_list = self._sort_op_recs(api_input_records)
 
             # Re-iterate until our bag of op-recs to process is empty.
             loop_counter = 0
@@ -479,7 +500,7 @@ class GraphBuilder(Specifiable):
                     # There are next records:
                     if len(op_rec.next) > 0:
                         # Push the op-record forward one step.
-                        for next_op_rec in sorted(op_rec.next, key=lambda rec: rec.id):  # type: DataOpRecord
+                        for next_op_rec in self._sort_op_recs(op_rec.next):  # type: DataOpRecord
                             # If not last op in this API-method ("done") -> continue.
                             # Otherwise, replace "done" with actual op.
                             if next_op_rec.op != "done":
@@ -502,7 +523,7 @@ class GraphBuilder(Specifiable):
                         new_op_records_to_process.update(op_rec.column.out_graph_fn_column.op_records)
 
                 # No sanity checks because meta graph was already built successfully.
-                new_op_records_list = sorted(new_op_records_to_process, key=lambda rec: rec.id)
+                new_op_records_list = self._sort_op_recs(new_op_records_to_process)
                 op_records_list = new_op_records_list
                 loop_counter += 1
         return subgraph_container
@@ -561,6 +582,7 @@ class GraphBuilder(Specifiable):
                                 format(op_rec_column.graph_fn.__name__)
                             )
                         num_return_values = len(ops[key])
+
                     # Un-split the results dict into a tuple of `num_return_values` slots.
                     un_split_ops = []
                     for i in range(num_return_values):
@@ -618,13 +640,14 @@ class GraphBuilder(Specifiable):
             assert op_rec_column.out_graph_fn_column is not None,\
                 "ERROR: DataOpRecordColumnFromGraphFn for in-column {} is None!".format(op_rec_column)
             out_graph_fn_column = op_rec_column.out_graph_fn_column
-            # Make sure the number of returned ops matches the number of op-records in the next column.
-            assert len(ops) == len(out_graph_fn_column.op_records), \
-                "ERROR: Number of returned values of graph_fn '{}/{}' ({}) does not match the number of op-records " \
-                "({}) reserved for the return values of the method!".format(
-                    op_rec_column.component.name, op_rec_column.graph_fn.__name__, len(ops),
-                    len(out_graph_fn_column.op_records)
-                )
+
+        # Make sure the number of returned ops matches the number of op-records in the next column.
+        assert len(ops) == len(out_graph_fn_column.op_records), \
+            "ERROR: Number of returned values of graph_fn '{}/{}' ({}) does not match the number of op-records " \
+            "({}) reserved for the return values of the method!".format(
+                op_rec_column.component.name, op_rec_column.graph_fn.__name__, len(ops),
+                len(out_graph_fn_column.op_records)
+            )
 
         # Determine the Spaces for each out op and then move it into the respective op and Space slot of the
         # out_graph_fn_column.
@@ -694,7 +717,7 @@ class GraphBuilder(Specifiable):
         # Step via `previous` through the graph backwards.
         while True:
             previous_op_rec = op_rec.previous
-            # Hit a graph_fn. Jump to incomign column.
+            # Hit a graph_fn. Jump to incoming column.
             if previous_op_rec is None:
                 assert isinstance(op_rec.column, DataOpRecordColumnFromGraphFn),\
                     "ERROR: If previous op-rec is None, column must be of type `DataOpRecordColumnFromGraphFn` " \
@@ -825,6 +848,10 @@ class GraphBuilder(Specifiable):
                                                                api_method_call[2] is not None else None
                 api_method_call = api_method_call[0]
 
+            if callable(api_method_call):
+                # Allow passing the function directly
+                api_method_call = api_method_call.__name__
+
             if api_method_call not in self.api:
                 raise RLGraphError("No API-method with name '{}' found!".format(api_method_call))
 
@@ -846,10 +873,19 @@ class GraphBuilder(Specifiable):
                         return_ops)
 
             for i, param in enumerate(params):
-                # TODO: What if len(params) < len(self.api[api_method][0])? Need to handle default API-method params also for the root-component (this one).
+                if param is None:
+                    assert len(self.api[api_method_call][0]) == i, \
+                        "ERROR: More input params given ({}) than expected ({}) for call to '{}'!". \
+                        format(len(params), len(self.api[api_method_call][0]), api_method_call)
+                    break
+
+                # TODO: What if len(params) < len(self.api[api_method][0])?
+                # Need to handle default API-method params also for the root-component (this one).
                 if len(self.api[api_method_call][0]) <= i:
-                    raise RLGraphError("API-method with name '{}' only has {} input parameters! You passed in "
-                                       "{}.".format(api_method_call, len(self.api[api_method_call][0]), len(params)))
+                    raise RLGraphError(
+                        "API-method with name '{}' only has {} input parameters! You passed in "
+                        "{}.".format(api_method_call, len(self.api[api_method_call][0]), len(params))
+                    )
 
                 placeholder = self.api[api_method_call][0][i].op  # 0=input op-recs; i=ith input op-rec
                 if isinstance(placeholder, ContainerDataOp):
@@ -898,6 +934,7 @@ class GraphBuilder(Specifiable):
         Executes a graph_fn in define by run mode.
 
         Args:
+            component (Component): Component this graph_fn is eecuted on.
             graph_fn (callable): Graph function to execute.
             options (dict): Execution options.
         Returns:
@@ -1041,7 +1078,7 @@ class GraphBuilder(Specifiable):
             # Check whether the Component is input-complete (and build already if it is).
             self.build_component_when_input_complete(component)
 
-        op_records_list = sorted(self.op_records_to_process, key=lambda rec: rec.id)
+        op_records_list = self._sort_op_recs(self.op_records_to_process)
         iterations = self._build(op_records_list)
 
         # Set execution mode in components to change `call` behaviour to direct function evaluation.
@@ -1068,6 +1105,21 @@ class GraphBuilder(Specifiable):
         """
         loop_counter = 0
         while len(op_records_list) > 0:
+            # In this iteration, do we still have API-method op-recs (which are part of columns that go into or come
+            # from API-methods).
+            have_api_method_recs = any(
+                isinstance(or_.column, (DataOpRecordColumnIntoAPIMethod, DataOpRecordColumnFromAPIMethod)) for or_ in
+                op_records_list
+            )
+            highest_nesting_of_callable_graph_fn_column = -1
+            if have_api_method_recs is False:
+                for or_ in op_records_list:
+                    if isinstance(or_.column, DataOpRecordColumnIntoGraphFn):
+                        col = or_.column
+                        if col.is_complete() and col.already_sent is False and \
+                                col.component.nesting_level > highest_nesting_of_callable_graph_fn_column:
+                            highest_nesting_of_callable_graph_fn_column = col.component.nesting_level
+
             # Collect op-recs to process in the next iteration.
             self.op_records_to_process = set()
 
@@ -1078,7 +1130,7 @@ class GraphBuilder(Specifiable):
                 # There are next records:
                 if len(op_rec.next) > 0:
                     # Push actual op and Space forward one op-rec at a time.
-                    for next_op_rec in sorted(op_rec.next, key=lambda rec: rec.id):  # type: DataOpRecord
+                    for next_op_rec in self._sort_op_recs(op_rec.next):  # type: DataOpRecord
                         # Assert that next-record's `previous` field points back to op_rec.
                         assert next_op_rec.previous is op_rec, \
                             "ERROR: Op-rec {} in meta-graph has {} as next, but {}'s previous field points to {}!". \
@@ -1088,33 +1140,42 @@ class GraphBuilder(Specifiable):
                             assert next_op_rec.op is None or is_constant(next_op_rec.op) or next_op_rec.op is op_rec.op
                             self.op_records_to_process.add(next_op_rec)
                         # Push op and Space into next op-record.
-                        next_op_rec.op = op_rec.op
-                        next_op_rec.space = op_rec.space
+                        # With op-instructions instructions?
+                        if next_op_rec.op_instructions is not None:
+                            mo = re.match(r'^key-lookup:(.+)$', next_op_rec.op_instructions)
+                            if mo:
+                                lookup_key = mo.group(1)
+                                next_op_rec.op = op_rec.op[lookup_key]
+                                next_op_rec.space = op_rec.space[lookup_key]
+                        # No instructions -> simply pass on.
+                        else:
+                            next_op_rec.op = op_rec.op
+                            next_op_rec.space = op_rec.space
 
-                        # Also push Space into possible API-method record if slot's Space is still None.
-                        if isinstance(op_rec.column, DataOpRecordColumnIntoAPIMethod):
-                            param_name = get_call_param_name(op_rec)
-                            component = op_rec.column.api_method_rec.component
+                            # Also push Space into possible API-method record if slot's Space is still None.
+                            if isinstance(op_rec.column, DataOpRecordColumnIntoAPIMethod):
+                                param_name = get_call_param_name(op_rec)
+                                component = op_rec.column.api_method_rec.component
 
-                            # Place Space for this input-param name (valid for all input params of same name even of
-                            # different API-method of the same Component).
-                            if component.api_method_inputs[param_name] is None or \
-                                    component.api_method_inputs[param_name] == "flex":
-                                component.api_method_inputs[param_name] = next_op_rec.space
-                            # For non-space agnostic Components: Sanity check, whether Spaces are equivalent.
-                            elif component.space_agnostic is False:
-                                generic_space = check_space_equivalence(
-                                    component.api_method_inputs[param_name], next_op_rec.space
-                                )
-                                # Spaces are not equivalent.
-                                if generic_space is False:
-                                    raise RLGraphError(
-                                        "ERROR: op-rec '{}' has Space '{}', but input-param '{}' already has Space "
-                                        "'{}'!".format(next_op_rec, next_op_rec.space, param_name,
-                                                       component.api_method_inputs[param_name])
+                                # Place Space for this input-param name (valid for all input params of same name even of
+                                # different API-method of the same Component).
+                                if component.api_method_inputs[param_name] is None or \
+                                        component.api_method_inputs[param_name] == "flex":
+                                    component.api_method_inputs[param_name] = next_op_rec.space
+                                # For non-space agnostic Components: Sanity check, whether Spaces are equivalent.
+                                elif component.space_agnostic is False:
+                                    generic_space = check_space_equivalence(
+                                        component.api_method_inputs[param_name], next_op_rec.space
                                     )
-                                # Overwrite both entries with the more generic Space.
-                                next_op_rec.space = component.api_method_inputs[param_name] = generic_space
+                                    # Spaces are not equivalent.
+                                    if generic_space is False:
+                                        raise RLGraphError(
+                                            "ERROR: op-rec '{}' has Space '{}', but input-param '{}' already has Space "
+                                            "'{}'!".format(next_op_rec, next_op_rec.space, param_name,
+                                                           component.api_method_inputs[param_name])
+                                        )
+                                    # Overwrite both entries with the more generic Space.
+                                    next_op_rec.space = component.api_method_inputs[param_name] = generic_space
 
                         # Did we enter a new Component? If yes, check input-completeness and
                         # - If op_rec.column is None -> We are at the very beginning of the graph (op_rec.op is a
@@ -1128,27 +1189,41 @@ class GraphBuilder(Specifiable):
                 # No next records:
                 # - Op belongs to a column going into a graph_fn.
                 elif isinstance(op_rec.column, DataOpRecordColumnIntoGraphFn):
-                    # If column complete AND has not been sent through the graph_fn yet -> Call the graph_fn.
-                    if op_rec.column.is_complete() and op_rec.column.already_sent is False:
+                    # Only call the GraphFn iff:
+                    # There are no more DataOpRecordColumnIntoAPIMethod ops in our list: We would like to hold off
+                    # any graph fn calls for as long as possible.
+                    # We don't want to run through a graph_fn, then have to call an API-method from within that graph_fn
+                    # and the component of that API-method is not input-/variable-complete yet.
+                    if have_api_method_recs:
+                        # Recycle this op-rec.
+                        self.op_records_to_process.add(op_rec)
+                    # There are other graph_fn columns that have a higher Component nesting_level and are
+                    # actually callable -> Call those first.
+                    elif highest_nesting_of_callable_graph_fn_column > op_rec.column.component.nesting_level:
+                        # Recycle this op-rec.
+                        self.op_records_to_process.add(op_rec)
+                    # GraphFn column must be complete AND has not been sent through the graph_fn yet.
+                    elif op_rec.column.is_complete() and op_rec.column.already_sent is False:
                         # Only call the graph_fn if the Component is already input-complete.
                         if op_rec.column.component.input_complete:
                             # Call the graph_fn with the given column and call-options.
                             self.run_through_graph_fn_with_device_and_scope(op_rec.column)
                             # Store all resulting op_recs (returned by the graph_fn) to be processed next.
                             self.op_records_to_process.update(op_rec.column.out_graph_fn_column.op_records)
-                        # Component not input-complete. Keep coming back with this op.
+                        # Component not input-complete. Recycle this op-rec.
                         else:
                             self.build_component_when_input_complete(op_rec.column.component)
                             self.op_records_to_process.add(op_rec)
                             if op_rec.column.component.input_complete is False:
                                 non_complete_components.add(op_rec.column.component.global_scope)
+                    # - There are still into-API-method-op-recs that should be handled first.
                     # - Op column is not complete yet: Discard this one (as others will keep coming in anyway).
                     # - Op column has already been sent (sibling ops may have arrived in same iteration).
-                # - Op belongs to a column coming from a graph_fn or an API-method, but the op is no longer used.
+                # else: - Op belongs to a column coming from a graph_fn or an API-method, but the op is no longer used.
                 # -> Ignore Op.
 
             # Sanity check, whether we are stuck.
-            new_op_records_list = sorted(self.op_records_to_process, key=lambda rec: rec.id)
+            new_op_records_list = self._sort_op_recs(self.op_records_to_process)
             if op_records_list == new_op_records_list:
                 # Probably deadlocked. Do a premature sanity check to report possible problems.
                 if loop_counter > self.max_build_iterations:
@@ -1188,3 +1263,32 @@ class GraphBuilder(Specifiable):
 
             loop_counter += 1
         return loop_counter
+
+    @staticmethod
+    def _sort_op_recs(recs):
+        """
+        Sorts op-recs according to:
+        - Give API-method calls priority over GraphFn calls (API-method call ops just have to be passed along without
+        worrying about input-/variable-completeness).
+        - Give deeper nested Components priority over shallower nested ones.
+        - Sort by op-rec ID to enforce determinism.
+
+        Note: We sort in reverse order, highest key-values first.
+
+        Args:
+            recs (Set[DataOpRecord]): The DataOpRecords to sort.
+
+        Returns:
+            list: The sorted op-recs.
+        """
+        def sorting_func(rec):
+            # Op-rec is a placeholder. Highest priority.
+            if rec.column is None:
+                return DataOpRecord.MAX_ID * 2 + rec.id
+            # API-methods have priority (over GraphFns).
+            elif isinstance(rec.column, DataOpRecordColumnIntoAPIMethod):
+                return DataOpRecord.MAX_ID + rec.id
+            # Deeper nested Components have priority. If same level, use op-rec's ID for determinism.
+            return rec.column.component.nesting_level + rec.id / DataOpRecord.MAX_ID
+
+        return sorted(recs, key=sorting_func, reverse=True)
