@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -77,6 +77,9 @@ class GraphExecutor(Specifiable):
 
         # Number of available GPUs and their names.
         self.gpus_enabled = None
+        # Whether to fake GPUs in case there are none available (in which case, we place everything on the CPU).
+        self.fake_gpus = None
+
         self.gpu_names = None
         self.used_devices = list()
         self.max_usable_gpus = 0
@@ -154,12 +157,21 @@ class GraphExecutor(Specifiable):
         """
         pass
 
-    def load_model(self, path=None):
+    def load_model(self, checkpoint_directory=None, checkpoint_path=None):
         """
-        Loads model from specified path location.
+        Loads model from specified path location using the following semantics:
+
+        If checkpoint directory and checkpoint path are given, attempts to find `checkpoint_path` as relative path from
+        `checkpoint_directory`.
+
+        If a checkpoint directory is given but no path (e.g. because timestep of checkpoint is not known in advance),
+        attempts to fetch latest check-point.
+
+        If no directory is given, attempts to fetch checkpoint from the full absolute path `checkpoint_path'.
 
         Args:
-            path (str): Path to checkpoint or model.
+            checkpoint_directory (str): Optional path to directory containing checkpoint(s).
+            checkpoint_path (str): Path to specific model checkpoint.
         """
         raise NotImplementedError
 
@@ -187,26 +199,26 @@ class GraphExecutor(Specifiable):
         """
         pass
 
-    def get_weights(self):
-        """
-        Returns all weights for computation graph of  this graph executor.
+    #def get_weights(self):
+    #    """
+    #    Returns all weights for computation graph of this graph executor.
 
-        Returns:
-            any: Weights for this graph..
-        """
-        return self.execute("_variables")
+    #    Returns:
+    #        any: Weights for this graph..
+    #    """
+    #    return self.execute("_variables")
 
-    def set_weights(self, weights):
-        """
-        Sets weights of the underlying computation graph..
+    #def set_weights(self, weights):
+    #    """
+    #    Sets weights of the underlying computation graph..
 
-        Args:
-            weights (any): Weights and optionally meta data to update depending on the backend.
+    #    Args:
+    #        weights (any): Weights and optionally meta data to update depending on the backend.
 
-        Raises:
-            ValueError if weights do not match graph weights in shapes and types.
-        """
-        self.execute(("sync", weights))
+    #    Raises:
+    #        ValueError if weights do not match graph weights in shapes and types.
+    #    """
+    #    self.execute(("sync", weights))
 
     def terminate(self):
         """
