@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,7 +22,7 @@ from six.moves import xrange as range_
 import re
 
 from rlgraph import get_backend
-from rlgraph.utils.util import dtype
+from rlgraph.utils.util import convert_dtype
 from rlgraph.utils.initializer import Initializer
 from rlgraph.spaces import Space
 
@@ -70,12 +70,10 @@ class BoxSpace(Space):
             assert isinstance(shape, (tuple, list)), "ERROR: `shape` must be None or a tuple/list."
             self._shape = tuple(shape)
 
+        # Determine the bounds.
         # False if bounds are individualized (each dimension has its own lower and upper bounds and we can get
         # the single values from self.low and self.high), or a tuple of the globally valid low/high values that apply
         # to all values in all dimensions.
-        self.global_bounds = None
-
-        # Determine the bounds.
         # 0D Space.
         if self._shape == ():
             assert isinstance(low, (int, float, bool))
@@ -166,8 +164,8 @@ class BoxSpace(Space):
 
             # Un-indent and just directly construct pytorch?
             if get_backend() == "pytorch" and is_input_feed:
-                # Convert to PyTorch tensor because PyTorch cannot use
-                return torch.zeros(shape)
+                # Convert to PyTorch tensors as a faux placehodler.
+                return torch.zeros(shape, dtype=convert_dtype(dtype=self.dtype, to="pytorch"))
             else:
                 # TODO also convert?
                 return var
@@ -176,7 +174,7 @@ class BoxSpace(Space):
             # TODO: re-evaluate the cutting of a leading '/_?' (tf doesn't like it)
             name = re.sub(r'^/_?', "", name)
             if is_input_feed:
-                return tf.placeholder(dtype=dtype(self.dtype), shape=shape, name=name)
+                variable = tf.placeholder(dtype=convert_dtype(self.dtype), shape=shape, name=name)
             else:
                 init_spec = kwargs.pop("initializer", None)
                 # Bools should be initializable via 0 or not 0.
@@ -188,11 +186,17 @@ class BoxSpace(Space):
                 else:
                     initializer = Initializer.from_spec(shape=shape, specification=init_spec).initializer
 
-                return tf.get_variable(
-                    name, shape=shape, dtype=dtype(self.dtype), initializer=initializer,
+                variable = tf.get_variable(
+                    name, shape=shape, dtype=convert_dtype(self.dtype), initializer=initializer,
                     collections=[tf.GraphKeys.GLOBAL_VARIABLES if local is False else tf.GraphKeys.LOCAL_VARIABLES],
                     **kwargs
                 )
+            # Add batch/time rank flags to the op.
+            if self.has_batch_rank:
+                variable._batch_rank = 0 if self.time_major is False else 1
+            if self.has_time_rank:
+                variable._time_rank = 1 if self.time_major is False else 0
+            return variable
 
     def zeros(self, size=None):
         return self.sample(size=size, fill_value=0)
