@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,11 +23,11 @@ import numpy as np
 
 from rlgraph.spaces.bool_box import BoolBox
 from rlgraph.spaces.box_space import BoxSpace
-from rlgraph.spaces.containers import ContainerSpace, Dict, Tuple
+from rlgraph.spaces.containers import Dict, Tuple
 from rlgraph.spaces.float_box import FloatBox
 from rlgraph.spaces.int_box import IntBox
 from rlgraph.spaces.text_box import TextBox
-from rlgraph.utils.util import RLGraphError, dtype, get_shape
+from rlgraph.utils.util import RLGraphError, convert_dtype, get_shape, LARGE_INTEGER
 
 if get_backend() == "pytorch":
     import torch
@@ -119,7 +119,9 @@ def get_space_from_op(op):
             raise RLGraphError("Cannot derive Space from non-allowed op ({})!".format(op))
         # A single numpy array.
         elif isinstance(op, np.ndarray):
-            return BoxSpace.from_spec(spec=dtype(str(op.dtype), "np"), shape=op.shape)
+            return BoxSpace.from_spec(spec=convert_dtype(str(op.dtype), "np"), shape=op.shape)
+        elif isinstance(op, list):
+            return try_space_inference_from_list(op)
         # No Space: e.g. the tf.no_op, a distribution (anything that's not a tensor).
         # PyTorch Tensors do not have get_shape so must check backend.
         elif hasattr(op, "dtype") is False or (get_backend() == "tf" and not hasattr(op, "get_shape")):
@@ -175,11 +177,11 @@ def get_space_from_op(op):
             # FloatBox
             if "float" in base_dtype_str:
                 return FloatBox(shape=shape, add_batch_rank=add_batch_rank, add_time_rank=add_time_rank,
-                                time_major=time_major, dtype=dtype(base_dtype, "np"))
+                                time_major=time_major, dtype=convert_dtype(base_dtype, "np"))
             # IntBox
             elif "int" in base_dtype_str:
                 return IntBox(shape=shape, add_batch_rank=add_batch_rank, add_time_rank=add_time_rank,
-                              time_major=time_major, dtype=dtype(base_dtype, "np"))
+                              time_major=time_major, dtype=convert_dtype(base_dtype, "np"))
             # a BoolBox
             elif "bool" in base_dtype_str:
                 return BoolBox(shape=shape, add_batch_rank=add_batch_rank, add_time_rank=add_time_rank,
@@ -196,6 +198,7 @@ def sanity_check_space(
         space, allowed_types=None, allowed_sub_types=None, non_allowed_types=None, non_allowed_sub_types=None,
         must_have_batch_rank=None, must_have_time_rank=None, must_have_batch_or_time_rank=False,
         must_have_categories=None, num_categories=None,
+        must_have_lower_limit=None, must_have_upper_limit=None,
         rank=None
 ):
     """
@@ -229,6 +232,9 @@ def sanity_check_space(
             `num_categories` rank must lie. Only valid for IntBoxes.
             None if it doesn't matter.
 
+        must_have_lower_limit (Optional[bool]): If not None, whether this Space must have a lower limit.
+        must_have_upper_limit (Optional[bool]): If not None, whether this Space must have an upper limit.
+
         rank (Optional[int,tuple]): An int or a tuple (min,max) range within which the Space's rank must lie.
             None if it doesn't matter.
 
@@ -298,6 +304,26 @@ def sanity_check_space(
                 raise RLGraphError("ERROR: Space {}({}) must have categories (globally valid value bounds)!".
                                    format("" if flat_key == "" else "'{}' ".format(flat_key), space))
 
+    if must_have_lower_limit is not None:
+        for flat_key, sub_space in flattened_space.items():
+            low = sub_space.low
+            if must_have_lower_limit is True and (low == -LARGE_INTEGER or low == float("-inf")):
+                raise RLGraphError("ERROR: Space {}({}) must have a lower limit, but has none!".
+                                   format("" if flat_key == "" else "'{}' ".format(flat_key), space))
+            elif must_have_lower_limit is False and (low != -LARGE_INTEGER and low != float("-inf")):
+                raise RLGraphError("ERROR: Space {}({}) must not have a lower limit, but has one ({})!".
+                                   format("" if flat_key == "" else "'{}' ".format(flat_key), space, low))
+
+    if must_have_upper_limit is not None:
+        for flat_key, sub_space in flattened_space.items():
+            high = sub_space.high
+            if must_have_upper_limit is True and (high != LARGE_INTEGER and high != float("inf")):
+                raise RLGraphError("ERROR: Space {}({}) must have an upper limit, but has none!".
+                                   format("" if flat_key == "" else "'{}' ".format(flat_key), space))
+            elif must_have_upper_limit is False and (high == LARGE_INTEGER or high == float("inf")):
+                raise RLGraphError("ERROR: Space {}({}) must not have a upper limit, but has one ({})!".
+                                   format("" if flat_key == "" else "'{}' ".format(flat_key), space, high))
+
     if rank is not None:
         if isinstance(rank, int):
             for flat_key, sub_space in flattened_space.items():
@@ -372,3 +398,35 @@ def check_space_equivalence(space1, space2):
     # TODO: time rank?
 
     return False
+
+
+def try_space_inference_from_list(list_op):
+    """
+    Attempts to infer shape space from a list op. A list op may be the result of fetching state from a Python
+    memory.
+
+    Args:
+        list_op (list): List with arbitrary sub-structure.
+
+    Returns:
+        Space: Inferred Space object represented by list.
+    """
+    if get_backend() == "pytorch":
+        batch_shape = len(list_op)
+        if batch_shape > 0:
+            # Try to infer more things by looking inside list.
+            elem = list_op[0]
+            if isinstance(elem, torch.Tensor):
+                list_type = elem.dtype
+                inner_shape = elem.shape
+                return BoxSpace.from_spec(spec=convert_dtype(list_type, "np"), shape=(batch_shape,) + inner_shape,
+                                          add_batch_rank=True)
+            elif isinstance(elem, list):
+                inner_shape = len(elem)
+                return BoxSpace.from_spec(spec=convert_dtype(float, "np"), shape=(batch_shape, inner_shape),
+                                          add_batch_rank=True)
+        else:
+            # Most general guess is a Float box.
+            return FloatBox(shape=(batch_shape,))
+    else:
+        raise ValueError("List inference should only be attempted on the Python backend.")
\ No newline at end of file
