@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -62,7 +62,7 @@ class TestActorComponents(unittest.TestCase):
         # Raw action layer output.
         expected_action_layer_output = np.matmul(
             expected_nn_output,
-            actor_component_params["actor-component/policy/action-adapter-0/action-layer/dense/kernel"]
+            actor_component_params["actor-component/policy/action-adapter-0/action-network/action-layer/dense/kernel"]
         )
         # Final actions (max-likelihood/greedy pick).
         expected_actions = np.argmax(expected_action_layer_output, axis=-1)
@@ -82,7 +82,7 @@ class TestActorComponents(unittest.TestCase):
         # Raw action layer output.
         expected_action_layer_output = np.matmul(
             expected_nn_output,
-            actor_component_params["actor-component/policy/action-adapter-0/action-layer/dense/kernel"]
+            actor_component_params["actor-component/policy/action-adapter-0/action-network/action-layer/dense/kernel"]
         )
         # No reshape necessary (simple action space), softmax to get probs.
         expected_action_probs = softmax(expected_action_layer_output)
@@ -162,7 +162,10 @@ class TestActorComponents(unittest.TestCase):
         preprocessor_spec = dict(
             type="dict-preprocessor-stack",
             preprocessors=dict(
-                a=[dict(type="convert_type", to_dtype="float"), dict(type="multiply", factor=0.5)]
+                a=[
+                    dict(type="convert_type", to_dtype="float"),
+                    dict(type="multiply", factor=0.5)
+                ]
             )
         )
         # Simple custom NN with dict input (splits into 2 streams (simple dense layers) and concats at the end).
@@ -194,7 +197,7 @@ class TestActorComponents(unittest.TestCase):
         # Raw action layer output.
         expected_action_layer_output = np.matmul(
             expected_nn_output,
-            actor_component_params["actor-component/policy/action-adapter-0/action-layer/dense/kernel"]
+            actor_component_params["actor-component/policy/action-adapter-0/action-network/action-layer/dense/kernel"]
         )
         # Final actions (max-likelihood/greedy pick).
         expected_actions = np.argmax(expected_action_layer_output, axis=-1)
