@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -84,6 +84,39 @@ class TestNeuralNetworks(unittest.TestCase):
         test.terminate()
 
     def test_lstm_nn(self):
+        # Space must contain batch dimension (otherwise, NNlayer will complain).
+        #units = 3
+        batch_size = 2
+        time_steps = 4
+        input_nodes = 2
+        input_space = FloatBox(shape=(input_nodes,), add_batch_rank=True, add_time_rank=True)
+        #internal_states_space = Tuple(FloatBox(shape=(units,)), FloatBox(shape=(units,)), add_batch_rank=True)
+
+        neural_net = NeuralNetwork.from_spec(config_from_path("configs/test_dense_to_lstm_nn.json"))
+
+        # Do not seed, we calculate expectations manually.
+        test = ComponentTest(component=neural_net, input_spaces=dict(
+            inputs=input_space
+        ))
+
+        # Batch of size=2, time-steps=3.
+        input_ = input_space.sample((batch_size, time_steps))
+
+        # Calculate output manually.
+        w0_value = test.read_variable_values(neural_net.variables["test-lstm-network/dense-layer/dense/kernel"])
+        b0_value = test.read_variable_values(neural_net.variables["test-lstm-network/dense-layer/dense/bias"])
+        lstm_w_value = test.read_variable_values(neural_net.variables["test-lstm-network/lstm-layer/lstm-cell/kernel"])
+        lstm_b_value = test.read_variable_values(neural_net.variables["test-lstm-network/lstm-layer/lstm-cell/bias"])
+
+        d0_out = dense_layer(input_, w0_value, b0_value)
+        lstm_out, last_internal_states = lstm_layer(d0_out, lstm_w_value, lstm_b_value, time_major=False)
+
+        expected = dict(output=lstm_out, last_internal_states=last_internal_states)
+        test.test(("apply", input_), expected_outputs=expected, decimals=5)
+
+        test.terminate()
+
+    def test_lstm_nn_with_custom_apply(self):
         # Space must contain batch dimension (otherwise, NNlayer will complain).
         units = 3
         batch_size = 2
