@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ from __future__ import division
 from __future__ import print_function
 
 import numpy as np
+from scipy.stats import beta
 import unittest
 
 from rlgraph.components.policies import Policy, SharedValueFunctionPolicy, DuelingPolicy
@@ -44,7 +45,8 @@ class TestPolicies(unittest.TestCase):
                 nn_input=state_space,
                 actions=action_space,
                 logits=flat_float_action_space,
-                probabilities=flat_float_action_space
+                probabilities=flat_float_action_space,
+                parameters=flat_float_action_space
             ),
             action_space=action_space
         )
@@ -58,7 +60,7 @@ class TestPolicies(unittest.TestCase):
 
         # Raw action layer output; Expected shape=(2,5): 2=batch, 5=action categories
         expected_action_layer_output = np.matmul(
-            expected_nn_output, policy_params["policy/action-adapter-0/action-layer/dense/kernel"]
+            expected_nn_output, policy_params["policy/action-adapter-0/action-network/action-layer/dense/kernel"]
         )
         test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
                   decimals=5)
@@ -125,6 +127,7 @@ class TestPolicies(unittest.TestCase):
                 nn_input=state_space,
                 actions=action_space,
                 probabilities=flat_float_action_space,
+                parameters=flat_float_action_space,
                 logits=flat_float_action_space
             ),
             action_space=action_space,
@@ -142,7 +145,7 @@ class TestPolicies(unittest.TestCase):
         # Raw action layer output; Expected shape=(3,3): 3=batch, 2=action categories + 1 state value
         expected_action_layer_output = np.matmul(
             expected_nn_output,
-            policy_params["shared-value-function-policy/action-adapter-0/action-layer/dense/kernel"]
+            policy_params["shared-value-function-policy/action-adapter-0/action-network/action-layer/dense/kernel"]
         )
         test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
                   decimals=5)
@@ -150,7 +153,7 @@ class TestPolicies(unittest.TestCase):
         # State-values: One for each item in the batch.
         expected_state_value_output = np.matmul(
             expected_nn_output,
-            policy_params["shared-value-function-policy/value-function-node/dense/kernel"]
+            policy_params["shared-value-function-policy/value-function-node/dense-layer/dense/kernel"]
         )
         test.test(("get_state_values", states), expected_outputs=dict(state_values=expected_state_value_output),
                   decimals=5)
@@ -196,10 +199,14 @@ class TestPolicies(unittest.TestCase):
         flat_float_action_space = FloatBox(shape=(4,), add_batch_rank=True, add_time_rank=True)
 
         # Policy with baseline action adapter AND batch-apply over the entire policy (NN + ActionAdapter + distr.).
+        network_spec = config_from_path("configs/test_lrelu_nn.json")
+        # Add folding to network.
+        network_spec["fold_time_rank"] = True
         shared_value_function_policy = SharedValueFunctionPolicy(
-            network_spec=config_from_path("configs/test_lrelu_nn.json"),
+            network_spec=network_spec,
+            action_adapter_spec=dict(unfold_time_rank=True),
             action_space=action_space,
-            batch_apply=True
+            value_unfold_time_rank=True
         )
         test = ComponentTest(
             component=shared_value_function_policy,
@@ -207,6 +214,7 @@ class TestPolicies(unittest.TestCase):
                 nn_input=state_space,
                 actions=action_space,
                 probabilities=flat_float_action_space,
+                parameters=flat_float_action_space,
                 logits=flat_float_action_space
             ),
             action_space=action_space,
@@ -224,22 +232,23 @@ class TestPolicies(unittest.TestCase):
 
         # Raw action layer output; Expected shape=(3,3): 3=batch, 2=action categories + 1 state value
         expected_action_layer_output = np.matmul(
-            expected_nn_output, policy_params["shared-value-function-policy/action-adapter-0/action-layer/dense/kernel"]
+            expected_nn_output, policy_params["shared-value-function-policy/action-adapter-0/action-network/"
+                                              "action-layer/dense/kernel"]
         )
-        expected_action_layer_output = np.reshape(expected_action_layer_output, newshape=(6, 4))
+        expected_action_layer_output = np.reshape(expected_action_layer_output, newshape=(2, 3, 4))
         test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
                   decimals=5)
-        expected_action_layer_output_unfolded = np.reshape(expected_action_layer_output, newshape=(2, 3, 4))
 
         # State-values: One for each item in the batch.
         expected_state_value_output = np.matmul(
             expected_nn_output,
-            policy_params["shared-value-function-policy/value-function-node/dense/kernel"]
+            policy_params["shared-value-function-policy/value-function-node/dense-layer/dense/kernel"]
         )
         expected_state_value_output_unfolded = np.reshape(expected_state_value_output, newshape=(2, 3, 1))
         test.test(("get_state_values", states), expected_outputs=dict(state_values=expected_state_value_output_unfolded),
                   decimals=5)
 
+        expected_action_layer_output_unfolded = np.reshape(expected_action_layer_output, newshape=(2, 3, 4))
         test.test(
             ("get_state_values_logits_probabilities_log_probs", states, ["state_values", "logits"]),
             expected_outputs=dict(
@@ -314,6 +323,7 @@ class TestPolicies(unittest.TestCase):
                 nn_input=nn_input_space,
                 actions=action_space,
                 probabilities=flat_float_action_space,
+                parameters=flat_float_action_space,
                 logits=flat_float_action_space
             ),
             action_space=action_space
@@ -385,3 +395,80 @@ class TestPolicies(unittest.TestCase):
         self.assertTrue(out["entropy"].dtype == np.float32)
         self.assertTrue(out["entropy"].shape == (3,))
 
+    def test_policy_for_bounded_continuous_action_space(self):
+        """
+        https://github.com/rlgraph/rlgraph/issues/43
+        """
+        nn_input_space = FloatBox(shape=(4,), add_batch_rank=True)
+        action_space = FloatBox(low=-1.0, high=1.0, shape=(1,), add_batch_rank=True)
+        # Double the shape for alpha/beta params.
+        action_space_parameters = FloatBox(shape=(2,), add_batch_rank=True)
+
+        policy = Policy(network_spec=config_from_path("configs/test_simple_nn.json"), action_space=action_space)
+        test = ComponentTest(
+            component=policy,
+            input_spaces=dict(
+                nn_input=nn_input_space,
+                actions=action_space,
+                logits=FloatBox(shape=(1,), add_batch_rank=True),
+                probabilities=FloatBox(add_batch_rank=True),
+                parameters=action_space_parameters,
+            ),
+            action_space=action_space
+        )
+
+        policy_params = test.read_variable_values(policy.variables)
+
+        # Some NN inputs.
+        nn_input = nn_input_space.sample(size=3)
+        # Raw NN-output.
+        expected_nn_output = np.matmul(nn_input, policy_params["policy/test-network/hidden-layer/dense/kernel"])
+        test.test(("get_nn_output", nn_input), expected_outputs=dict(output=expected_nn_output))
+
+        # Raw action layer output.
+        expected_raw_logits = np.matmul(
+            expected_nn_output, policy_params["policy/action-adapter-0/action-network/action-layer/dense/kernel"]
+        )
+        test.test(("get_action_layer_output", nn_input), expected_outputs=dict(output=expected_raw_logits),
+                  decimals=5)
+
+        # Parameter (alpha/betas).
+        expected_parameters_output = np.log(np.exp(expected_raw_logits) + 1.0) + 1.0
+        test.test(("get_logits_parameters_log_probs", nn_input, ["logits", "parameters"]), expected_outputs=dict(
+            logits=expected_raw_logits, parameters=expected_parameters_output
+        ), decimals=5)
+
+        print("Params: {}".format(expected_parameters_output))
+
+        actions = test.test(("get_action", nn_input))["action"]
+        self.assertTrue(actions.dtype == np.float32)
+        self.assertGreaterEqual(actions.min(), -1.0)
+        self.assertLessEqual(actions.max(), 1.0)
+        self.assertTrue(actions.shape == (3, 1))
+
+        # Action log-probs.
+        actions_scaled_back = (actions + 1.0) / 2.0
+        expected_action_log_prob_output = np.log(beta.pdf(actions_scaled_back, expected_parameters_output[:, 1], expected_parameters_output[:, 0]))
+        expected_action_log_prob_output = np.array([[expected_action_log_prob_output[0][0]], [expected_action_log_prob_output[1][1]], [expected_action_log_prob_output[2][2]]])
+        test.test(("get_action_log_probs", [nn_input, actions]),
+                  expected_outputs=dict(action_log_probs=expected_action_log_prob_output,
+                                        logits=expected_raw_logits), decimals=5)
+
+        # Stochastic sample.
+        actions = test.test(("get_stochastic_action", nn_input))["action"]
+        self.assertTrue(actions.dtype == np.float32)
+        self.assertGreaterEqual(actions.min(), -1.0)
+        self.assertLessEqual(actions.max(), 1.0)
+        self.assertTrue(actions.shape == (3, 1))
+
+        # Deterministic sample.
+        actions = test.test(("get_deterministic_action", nn_input))["action"]
+        self.assertTrue(actions.dtype == np.float32)
+        self.assertGreaterEqual(actions.min(), -1.0)
+        self.assertLessEqual(actions.max(), 1.0)
+        self.assertTrue(actions.shape == (3, 1))
+
+        # Distribution's entropy.
+        entropy = test.test(("get_entropy", nn_input))["entropy"]
+        self.assertTrue(entropy.dtype == np.float32)
+        self.assertTrue(entropy.shape == (3, 1))
