@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -133,9 +133,11 @@ def parse_execution_spec(execution_spec):
                 gpus_enabled=False,
                 # If yes, how many GPUs are to be used?
                 max_usable_gpus=0,
+                # If True, use `max_usable_gpus` fake-GPUs (CPU) iff no GPUs are available.
+                fake_gpus_if_necessary=False,
                 # Specify specific CUDA devices to be used, e.g. gpu 0 and 2 = [0, 2].
                 # If None, we use CUDA devices [0, max_usable_gpus - 1]
-                enable_cuda_devices=None,
+                cuda_devices=None,
                 # Fraction of the overall amount of memory that each visible GPU should be allocated.
                 per_process_gpu_memory_fraction=None,
                 # If True, not all memory will be allocated which is relevant on shared resources.
@@ -197,7 +199,7 @@ def parse_execution_spec(execution_spec):
                 max_usable_gpus=0,
                 # Specify specific CUDA devices to be used, e.g. gpu 0 and 2 = [0, 2].
                 # If None, we use CUDA devices [0, max_usable_gpus - 1]
-                enable_cuda_devices=None
+                cuda_devices=None
             ),
             # Device placement settings.
             device_strategy="default",
@@ -222,10 +224,15 @@ def parse_observe_spec(observe_spec):
     Returns:
         dict: The sanitized observe_spec dict.
     """
+    if observe_spec and "buffer_enabled" not in observe_spec:
+        # Set to true if a buffer size is given and > 0, but flag is not given.
+        if "buffer_size" in observe_spec and observe_spec["buffer_size"] > 0:
+            observe_spec["buffer_enabled"] = True
+
     # If no spec given.
     default_spec = dict(
         # Do we buffer observations in python before sending them through the graph?
-        buffer_enabled=True,
+        buffer_enabled=False,
         # Fill buffer with n records before sending them through the graph.
         buffer_size=100,  # only if buffer_enabled=True
         # Set to > 1 if we want to post-process buffered values for n-step learning.
