@@ -21,7 +21,7 @@ import inspect
 import numpy as np
 
 from rlgraph.spaces.space_utils import get_space_from_op
-from rlgraph.utils import dtype
+from rlgraph.utils import convert_dtype
 from rlgraph.utils.rlgraph_errors import RLGraphError, RLGraphAPICallParamError
 from rlgraph.utils.ops import FlattenedDataOp, flatten_op, unflatten_op, is_constant
 
@@ -30,23 +30,33 @@ class DataOpRecord(object):
     """
     A simple wrapper class for a DataOp carrying the op itself and some additional information about it.
     """
+    # The current ID value.
     _ID = -1
+    # The highest allowed ID (can be changed to any higher value; used e.g. to compute sorting keys for
+    # build-prioritization).
+    MAX_ID = 1e6
 
-    def __init__(self, op=None, column=None, position=None, kwarg=None, space=None, previous=None, next_=None):
+    def __init__(self, op=None, column=None, position=None, kwarg=None, space=None, previous=None, next_record=None):
         """
         Args:
             op (Optional[DataOp]): The optional DataOp to already store in this op-rec.
             column (DataOpRecordColumn): The DataOpRecordColumn to which this op-rec belongs.
             position (Optional[int]): An optional position (index) for this op inside `column`.
+
             kwarg (Optional[str]): The keyword with which to call the API-method if this op-rec is not a positional
                 arg.
+
             space (Optional[Space]): The Space of `op` if already known at construction time. Will be poulated
                 later (during build phase) if not.
+
             next\_ (Optional(Set[DataOpRecord],DataOpRecord)): The next op-record or set of op-records.
             previous (Optional(DataOpRecord)): The previous op-record.
         """
         self.id = self.get_id()
         self.op = op
+        # Some instruction on how to derive the `op` property of this record (other than just: pass along).
+        # e.g. "key-lookup: [some key]" if previous op is a DataOpDict.
+        self.op_instructions = None
         # Whether the op in this record is one of the last in the graph (a core API-method returned op).
         self.is_terminal_op = False
 
@@ -58,19 +68,38 @@ class DataOpRecord(object):
         self.space = space
 
         # Set of (op-col ID, slot) tuples that are connected from this one.
-        self.next = next_ if isinstance(next_, set) else ({next_} if next_ is not None else set())
+        self.next = next_record if isinstance(next_record, set) else ({next_record}
+                                                                      if next_record is not None else set())
         # The previous op that lead to this one.
         self.previous = previous
 
     @staticmethod
     def get_id():
         DataOpRecord._ID += 1
+        if DataOpRecord._ID >= DataOpRecord.MAX_ID:
+            raise RLGraphError("Maximum number of op-rec IDs reached! Simply hard-increase `DataOpRecord.MAX_ID`.")
         return DataOpRecord._ID
 
     @staticmethod
     def reset():
         DataOpRecord._ID = -1
 
+    def __getitem__(self, key):
+        """
+        Returns new DataOpRecordColumn with
+
+        Args:
+            key (str): The lookup key
+
+        Returns:
+
+        """
+        column = DataOpRecordColumn(
+            self.column.component, args=[self]
+        )
+        column.op_records[0].op_instructions = "key-lookup:{}".format(key)
+        return column.op_records[0]
+
     def __str__(self):
         return "DataOpRec(id={} {}{})".format(
             self.id,"pos="+str(self.position) if self.kwarg is None else "kwarg="+self.kwarg,
@@ -110,7 +139,7 @@ class DataOpRecordColumn(object):
                     elif args[i] is not None:
                         op = args[i]
                         if is_constant(op) and not isinstance(op, np.ndarray):
-                            op = np.array(op, dtype=dtype(type(op), "np"))
+                            op = np.array(op, dtype=convert_dtype(type(op), "np"))
                         op_rec.op = op
                         op_rec.space = get_space_from_op(op)
                         component.constant_op_records.add(op_rec)
@@ -131,7 +160,7 @@ class DataOpRecordColumn(object):
                     elif value is not None:
                         op = value
                         if is_constant(op):
-                            op = np.array(op, dtype=dtype(type(op), "np"))
+                            op = np.array(op, dtype=convert_dtype(type(op), "np"))
                         op_rec.op = op
                         op_rec.space = get_space_from_op(op)
                         component.constant_op_records.add(op_rec)
@@ -271,9 +300,9 @@ class DataOpRecordColumnIntoGraphFn(DataOpRecordColumn):
         if len(flattened) > 1:
             # Loop through the non-first ones and make sure all keys match vs the first one.
             for other in flattened[1:]:
-                iter_ = iter(other)
+                other_arg_iter = iter(other)
                 for key, value in flattened[0]:
-                    k_other, v_other = next(iter_)
+                    k_other, v_other = next(other_arg_iter)
                     if key != k_other:  # or get_shape(v_other) != get_shape(value):
                         raise RLGraphError("ERROR: Flattened ops have a key mismatch ({} vs {})!".format(key, k_other))
 
@@ -488,8 +517,8 @@ def get_call_param_name(op_rec):
             else:
                 if api_method_rec.kwargs_name is None:
                     raise RLGraphAPICallParamError(
-                        "ERROR: API-method '{}' has no **kwargs, but op-rec {} indicates that it does!".
-                        format(api_method_rec.name, op_rec.id)
+                        "ERROR: API-method '{}' has no **kwargs, but op-rec {} indicates that it has kwarg '{}'!".
+                        format(api_method_rec.name, op_rec.id, op_rec.kwarg)
                     )
                 param_name = api_method_rec.kwargs_name + "[{}]".format(op_rec.kwarg)
     # There are *kwargs in the signature.
