@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph authors. All Rights Reserved.
+# Copyright 2018/2019 The RLgraph authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,3 +39,22 @@ class RLGraphBuildError(RLGraphError):
     components.
     """
     pass
+
+
+class RLGraphInputIncompleteError(RLGraphError):
+    """
+    Raised if the build of a model cannot
+    """
+    def __init__(self, msg, component):
+        super(RLGraphInputIncompleteError, self).__init__(msg)
+        self.component = component
+
+
+class RLGraphVariableIncompleteError(RLGraphError):
+    """
+
+    """
+    def __init__(self, msg, component):
+        super(RLGraphVariableIncompleteError, self).__init__(msg)
+        self.component = component
+
